/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.commerce.api;

import java.util.List;
import java.util.Locale;
import java.util.Map;

import com.adobe.cq.commerce.api.promotion.VoucherInfo;
import org.apache.commons.collections.Predicate;

import com.adobe.cq.commerce.api.promotion.PromotionInfo;
import com.adobe.cq.commerce.api.promotion.Voucher;

/**
 * The <code>CommerceSession</code> provides anonymous or authenticated access to a (possibly remote)
 * commerce server.
 *
 * <p>Session persistence is divided into two main parts: the cart entries (each one modeled by the
 * <code>CartEntry</code> interface), and the various order properties (collectively modeled by a
 * <code>Map&lt;String, Object></code>).</p>
 *
 * <p>In theory the various order properties could be updated as required via one (or more) calls to
 * <code>updateOrder()</code>, and then the order submitted via an empty call to <code>placeOrder()</code>.
 * However, in practice, security constraints (such as being forbidden from storing CCV values)
 * necessitate passing some of the order's properties in the final (<code>placeOrder()</code>) call.
 *
 * <p>It's up to the individual implementation to determine how many check-out steps there should be,
 * and how much information to include (only) in the final call.</p>
 */
public interface CommerceSession {

    /**
     * Logs the current user out.
     */
    public void logout() throws CommerceException {
	}

    /**
     * Forces a certain locale for the current user. Prices will be returned according the locale's
     * currency, if available.
     */
    public void setUserLocale(Locale locale) {
	}

    /**
     * Return the forced user locale or <code>null</code>, if no locale was set before.
     */
    public Locale getUserLocale() {
		return null;
	}

    /**
     * Get a list of all detailed price infos for a product  See {@link PriceInfo} for more details.
     */
    public List<PriceInfo> getProductPriceInfo(Product product) throws CommerceException {
		return null;
	}

    /**
     * Get a list of detailed price infos for a product filtered by the provided predicate
     * (usually a {@link com.adobe.cq.commerce.common.PriceFilter}).
     *
     * <p>See {@link PriceInfo} for more details.</p>
     *
     * @param product The product.
     * @param filter An optional predicate used for filtering the {@link PriceInfo}s.
     *               Pass <code>null</code> to fetch all {@link PriceInfo}s.
     * @return A list of all {@link PriceInfo}s which match the provided predicate.
     */
    public List<PriceInfo> getProductPriceInfo(Product product, Predicate filter) throws CommerceException {
		return null;
	}

    /**
     * Shortcut for getting a formatted string for the first applicable price for a product as
     * defined by the provided predicate (usually a {@link com.adobe.cq.commerce.common.PriceFilter}).
     *
     * <p>See {@link PriceInfo} for more details.</p>
     *
     * @param product The product
     * @param filter An optional predicate used for filtering the {@link PriceInfo}s.
     *               Pass <code>null</code> to fetch all {@link PriceInfo}s.
     * @return A formatted price string.
     */
    public String getProductPrice(Product product, Predicate filter) throws CommerceException {
		return null;
	}

    /**
     * Shortcut for getting a formatted string for the first price for a product.
     *
     * <p>See {@link PriceInfo} for more details.</p>
     *
     * @param product The product.
     * @return A formatted price string.
     */
    public String getProductPrice(Product product) throws CommerceException {
		return null;
	}

    /**
     * Get the number of cart entries.
     */
    public int getCartEntryCount() throws CommerceException {
		return 0;
	}

    /**
     * Get the cart entries as a <code>List</code>.
     */
    public List<CartEntry> getCartEntries() throws CommerceException {
		return null;
	}

    /**
     * <code>CartEntry</code> represents shopping cart line-items with a certain product and quantity.
     */
    public interface CartEntry {
        /**
         * Get the (0-based) index of the entry in the cart.
         */
        public int getEntryIndex() {
			return 0;
		}

        /**
         * Get the product.
         */
        public Product getProduct() throws CommerceException {
			return null;
		}

        /**
         * Get the quantity.
         */
        public int getQuantity() {
			return 0;
		}

        /**
         * <p>Get a list of detailed price info objects for a cart entry filtered by the provided
         * predicate (usually a {@link com.adobe.cq.commerce.common.PriceFilter}).</p>
         *
         * <p>Current components expect price infos with (at least) the following classifiers, in
         * this order:</p>
         * <ol>
         *     <li>"LINE" (line subtotal, generally without tax)</li>
         *     <li>"UNIT" (unit price, generally without tax)</li>
         * </ol>
         *
         * <p><strong>Example:</strong> to get the unit price of a cart entry, you could use the
         * following statement:</p>
         * <pre>
         * {@code
         * cartEntry.getPriceInfo(new PriceFilter("UNIT"));
         * }
         * </pre>
         *
         * @param filter An optional predicate used for filtering the {@link PriceInfo}s.
         *               Pass <code>null</code> to fetch all {@link PriceInfo}s.
         * @return a list of prices, can be empty if no price matches.
         * @see com.adobe.cq.commerce.common.PriceFilter
         */
        public List<PriceInfo> getPriceInfo(Predicate filter) throws CommerceException {
			return null;
		}

        /**
         * Shortcut for getting a formatted string of the first matching price.
         * @see #getPriceInfo(Predicate)
         *
         * @param filter An optional predicate used for filtering the {@link PriceInfo}s.
         * @return Formatted price string of first matching price
         */
        public String getPrice(Predicate filter) throws CommerceException {
			return null;
		}

        /**
         * Get a cart entry property with the given type.
         *
         * @param name The property's name
         * @param type The property's type
         * @return The property or <code>null</code> if it doesn't exist
         * or can not be converted to the given type.
         */
        public <T> T getProperty(String name, Class<T> type) {
			return null;
		}

        /**
         * @deprecated since 5.6, use {@link #getPrice} instead.
         */
        @Deprecated
        public String getUnitPrice() {
			return null;
		}
        /**
         * @deprecated since 5.6, use {@link #getPrice} instead.
         */
        @Deprecated
        public String getPreTaxPrice() {
			return null;
		}
        /**
         * @deprecated since 5.6, use {@link #getPrice} instead.
         */
        @Deprecated
        public String getTax() {
			return null;
		}
        /**
         * @deprecated since 5.6, use {@link #getPrice} instead.
         */
        @Deprecated
        public String getTotalPrice() {
			return null;
		}

    }

    /**
     * <p>Get detailed price information about the cart filtered by the provided predicate
     * (usually a {@link com.adobe.cq.commerce.common.PriceFilter}).</p>
     *
     * <p>Current components expect the following prices, in this order:</p>
     * <ol>
     *     <li>"CART" (cart subtotal, generally pre-tax)</li>
     *     <li>"ORDER" (order total, generally including tax and shipping)</li>
     * </ol>
     *
     * <p><strong>Examples:</strong></p>
     * <pre>
     * {@code
     * // get the cart subtotal
     * session.getCartPrice(null);
     * // get cart total
     * session.getCartPrice(new PriceFilter("POST_TAX"));
     * // get order tax
     * session.getCartPrice(new PriceFilter("ORDER", "TAX"));
     * }
     * </pre>
     *
     * @param filter An optional predicate used for filtering the {@link PriceInfo}s.
     *               Pass <code>null</code> to fetch all {@link PriceInfo}s.
     * @return a list of prices, can be empty if no price matches.
     * @throws CommerceException
     */
    public List<PriceInfo> getCartPriceInfo(Predicate filter) throws CommerceException {
		return null;
	}

    /**
     * Shortcut for getting a formatted string of the first applicable price, according to the filter.
     * @see #getCartPriceInfo(Predicate)
     *
     * @param filter An optional predicate used for filtering the {@link PriceInfo}s.
     *               Pass <code>null</code> to fetch all {@link PriceInfo}s.
     * @return Formatted price string of first matching price
     * @throws CommerceException
     */
    public String getCartPrice(Predicate filter) throws CommerceException {
		return null;
	}

    /**
     * Add a new entry to the cart.
     *
     * @param product The product to add
     * @param quantity The number of items to add
     * @throws CommerceException
     */
    public void addCartEntry(Product product, int quantity) throws CommerceException {
	}
    
    /**
     * Modify quantity of an existing cart entry.
     *
     * @param entryNumber The cart entry's number
     * @param quantity The new quantity
     * @throws CommerceException
     */
    public void modifyCartEntry(int entryNumber, int quantity) throws CommerceException {
	}

    /**
     * Delete an existing cart entry.
     *
     * @param entryNumber The cart entry's number
     * @throws CommerceException
     */
    public void deleteCartEntry(int entryNumber) throws CommerceException {
	}

    /**
     * Apply a voucher to this session's cart.
     * 
     * @param code the voucher's code
     * @throws CommerceException
     */
    public void addVoucher(String code) throws CommerceException {
	}
    
    /**
     * Remove a voucher that was previously added with {@link #addVoucher(String)}.
     * 
     * @param code the voucher's code
     * @throws CommerceException
     */
    public void removeVoucher(String code) throws CommerceException {
	}

    /**
     * Get a list of vouchers that were added to this cart via {@link #addVoucher(String)}, or that
     * were assigned by the commerce engine.
     *
     * @throws CommerceException
     */
    public List<VoucherInfo> getVoucherInfos() throws CommerceException {
		return null;
	}

    /**
     * Client-side promotion resolution allows a ClientContext store (the CartMgr in the OOTB
     * implementation) to resolve promotions and then adjust the current CommerceSession via
     * {@link #addPromotion(String)} and {@link #removePromotion(String)}.
     *
     * In contrast, if a CommerceSession wishes to resolve its own promotions on the server, it
     * should return <code>false</code> here and need only implement {@link #getPromotions()} to
     * return the resolved promotion info.
     * @return <code>true</code> if underlying implementation has support
     */
    public boolean supportsClientsidePromotionResolution() {
		return false;
	}

    /**
     * Apply a promotion to this session's cart.
     *
     * @param path A string uniquely identifying the promotion (generally a path)
     * @throws CommerceException
     */
    public void addPromotion(String path) throws CommerceException {
	}

    /**
     * Remove a promotion.
     *
     * @param path A string uniquely identifying the promotion (generally a path)
     * @throws CommerceException
     */
    public void removePromotion(String path) throws CommerceException {
	}

    /**
     * Get a list of promotions that are active for this cart.  Promotions might have been added
     * via {@link #addPromotion(String)}, or might have been assigned by the commerce engine.
     *
     * @throws CommerceException
     */
    public List<PromotionInfo> getPromotions() throws CommerceException {
		return null;
	}

    /**
     * Get a unique ID which identifies the current cart/order.  Note that the semantics of this
     * ID are highly implementation-dependent.  Hybris, for example, will return null until the
     * first product is added to the cart.  Other systems may have different semantics.
     *
     * @return A unique ID
     * @throws CommerceException
     */
    public String getOrderId() throws CommerceException {
		return null;
	}

    /**
     * Update a subset of the current order details.  Entries in the <code>Map</code> which do not
     * yet exist will be added to the order, while entries that already exist will be updated.
     *
     * @param delta A <code>Map</code> of name/value pairs
     * @throws CommerceException
     */
    public void updateOrder(Map<String, Object> delta) throws CommerceException {
	}

    /**
     * Get the current order details.
     *
     * @return A <code>Map</code> of name/value pairs
     * @throws CommerceException
     */
    public Map<String, Object> getOrder() throws CommerceException {
		return null;
	}

    /**
     * Submit the current order.  Note that the semantics of submitting an order are somewhat
     * implementation-dependent.
     *
     * @param orderDetailsDelta An optional set of order details to update before submission.
     *                          Useful for passing properties which cannot be stored, such as
     *                          the CCV.
     * @throws CommerceException
     */
    public void placeOrder(Map<String, Object> orderDetailsDelta) throws CommerceException {
	}

    /**
     * Get a list of previously placed orders.
     *
     * @param predicate An optional implementation-specific predicate name.  Implementations
     *                  should support at least {@link CommerceConstants#OPEN_ORDERS_PREDICATE}.
     * @param pageNumber the requested page number of the list of placed orders
     * @param pageSize the number of orders in a page
     * @param sortId the identifier of the sorting used to sort the list of placed orders
     *
     * @return a {@code PlacedOrderResult} object which contains the list of implementation-specific order summary info,
     * the pagination and sorting info of the returned orders. If {@code PaginationInfo} is null in the returned object
     * then the commerce implementation doesn't support pagination. If the returned object does not contain any
     * {@code CommerceSort} object then the returned orders are unsorted.
     *
     * @throws CommerceException
     */
    public PlacedOrderResult getPlacedOrders(String predicate, int pageNumber, int pageSize, String sortId) throws CommerceException {
		return null;
	}

    /**
     * Fetch a specific (previously-placed) order.
     *
     * @param orderId An implementation-specific id identifying the placed order
     * @throws CommerceException
     */
    public PlacedOrder getPlacedOrder(String orderId) throws CommerceException {
		return null;
	}


    /*
     * ==================================================================================================
     * Deprecated methods.  For backwards-compatibility only.
     * ==================================================================================================
     */

    /**
     * Get formatted price for a product.
     * @deprecated since 5.6, use {@link #getProductPrice(Product)}
     */
    @Deprecated
    public String getPriceInfo(Product product) throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #getCartPrice(Predicate)} instead.
     */
    @Deprecated
    public String getCartPreTaxPrice() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #getCartPrice(Predicate)} instead.
     */
    @Deprecated
    public String getCartTax() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #getCartPrice(Predicate)} instead.
     */
    @Deprecated
    public String getCartTotalPrice() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #getCartPrice(Predicate)} instead.
     */
    @Deprecated
    public String getOrderShipping() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #getCartPrice(Predicate)} instead.
     */
    @Deprecated
    public String getOrderTotalTax() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #getCartPrice(Predicate)} instead.
     */
    @Deprecated
    public String getOrderTotalPrice() throws CommerceException {
		return null;
	}

    /**
     * Get a list of vouchers that were added to this cart via {@link #addVoucher(String)}.
     * @deprecated since 5.6.200; use {@link #getVoucherInfos()} instead
     */
    @Deprecated
    public List<Voucher> getVouchers() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #updateOrder} instead.
     */
    @Deprecated
    public void updateOrderDetails(Map<String, String> delta) throws CommerceException {
	}

    /**
     * @deprecated since 5.6, use {@link #getOrder} instead.
     */
    @Deprecated
    public Map<String, String> getOrderDetails() throws CommerceException {
		return null;
	}

    /**
     * @deprecated since 5.6, use {@link #placeOrder} instead.
     */
    @Deprecated
    public void submitOrder(Map<String, String> orderDetailsDelta) throws CommerceException {
	}

}
