/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.calendar;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.TimeZone;

import net.fortuna.ical4j.model.TimeZoneRegistry;
import net.fortuna.ical4j.util.Dates;
import net.fortuna.ical4j.util.TimeZones;

import com.adobe.cq.social.calendar.impl.ICalEventMapper;
import com.adobe.cq.social.calendar.impl.JcrTimeZoneRegistry;

/**
 * Utility class for calendar and date conversions, time zone handling, etc.
 * @since 5.6
 */
public abstract class CalendarUtil {

    /**
     * Represents the "Etc/UTC" timezone with no offset at all.
     */
    public static final TimeZone UTC = ICalEventMapper.UTC_TIMEZONE;

    /**
     * Standard Olson timezone database ID for the UTC timezone.
     */
    public static final String UTC_ID = TimeZones.UTC_ID;

    /**
     * The {@link SimpleDateFormat}-style pattern for the Javascript / ECMA date format string.
     * <p>
     * Note that this is the variant without commata after weekday and after the date, which seems to be more common.
     */
    public static final String ECMA_DATE_PATTERN = "EEE MMM dd yyyy HH:mm:ss 'GMT'Z";

    /**
     * The {@link SimpleDateFormat}-style pattern for the Javascript / ECMA date format string when only a pure date
     * without time is to be transferred.
     */
    public static final String ECMA_DATE_ONLY_PATTERN = "EEE MMM dd yyyy";

    /**
     * Returns a new {@link SimpleDateFormat} that parses and formats date strings for use with Javascript (ECMA
     * Script) in browsers. The output can be used in {@code new Date(string)} in Javascript. An example ECMA date
     * string is: {@code Wed Aug 27 2008 23:59:59 GMT+0200}.
     * <p>
     * When {@code isDate == true}, the format will only print the date part and not include the time and timezone
     * parts. For example: {@code Wed Aug 27
     * 2008}.
     * <p>
     * The {@code timeZoneID} can be optionally given to denote the timezone in which the string should be formatted.
     * For example, an UTC date such as {@code Wed Aug 27 2008 10:00:00 GMT+0000} will result in:
     * <ul>
     * <li>{@code Wed Aug 27 2008 05:00:00 GMT-0500} for a timezone with {@code -5h} offset, eg. "America/New_York"</li>
     * <li>{@code Wed Aug 27 2008 11:00:00 GMT+0100} for a timezone with {@code +1h} offset, eg. "Europe/Berlin"</li>
     * </ul>
     * If {@code timeZoneID == null}, the UTC timezone will be used. When {@code isDate == true}, it will be ignored.
     * <p>
     * The locale used will always be US as that seems to be the string format that all browsers can parse with
     * <code>new Date(string)</code>. The patterns used are {@link #ECMA_DATE_PATTERN} and
     * {@link #ECMA_DATE_ONLY_PATTERN} (when {@code isDate == true}).
     * @param isDate whether only the date part should be formatted
     * @param timeZoneID a time zone ID such as "Europe/Berlin" (optional, can be {@code null}, in which case UTC is
     *            used; ignored when {@code isDate == true})
     * @return a {@link SimpleDateFormat} for the ECMA date format
     */
    public static SimpleDateFormat getEcmaDateFormat(final boolean isDate, final String timeZoneID) {
		return null;
	}

    private static TimeZoneRegistry tzRegistry;

    /**
     * Returns a timezone by it's Olson timezone database ID, using the internal timezone registry instead of the
     * built-in Java timezones {@link TimeZone#getTimeZone(String)}.
     * @param tzID timezone ID, eg. "Europe/Berlin"
     * @return a {@link TimeZone} object or <code>null</code> if the id was not found or was null
     */
    public static TimeZone getTimeZone(final String tzID) {
		return null;
	}

    /**
     * Shifts a calendar into the given timezone. This keeps the same UTC millis, but changes the representation to be
     * in the other timezone. This is a safe replacement for using {@link Calendar#setTimeZone(TimeZone)}.
     * <p>
     * For example, a date like <code>2009-08-10 12:00 +02:00</code> with a target timezone <code>-02:00</code> would
     * result in <code>2009-08-10 08:00 -02:00</code>.
     * @param cal input date in another timezone
     * @param timeZone target timezone
     * @return a new calendar object with the given date shifted into the given timezone
     */
    public static Calendar shiftTimeZone(final Calendar cal, final TimeZone timeZone) {
		return null;
	}

    /**
     * Shifts the calendar into the UTC timezone (no timezone offset). This is the same as calling
     * {@link #shiftTimeZone(Calendar, TimeZone)} with the "Etc/UTC" timezone.
     * <p>
     * For example, a date like <code>2009-08-10 12:00 +02:00</code> would result in
     * <code>2009-08-10 10:00 +00:00</code>.
     * @param cal input date in another timezone
     * @return a new calendar object with the given date shifted into the UTC timezone
     */
    public static Calendar shiftToUTC(final Calendar cal) {
		return null;
	}

    /**
     * Returns a new calendar that has exactly the same date and time parts as the given original calendar, but with
     * the new timezone. This will not adjust the time like {@link #shiftTimeZone(Calendar, TimeZone)} or
     * {@link Calendar#setTimeZone(java.util.TimeZone)} would do, but instead replaces it. This keeps the date and
     * time parts the same, and effectively changes the UTC millis.
     * <p>
     * For example, a date like <code>2009-08-10 12:00 +02:00</code> with a target timezone <code>-02:00</code> would
     * result in <code>2009-08-10 12:00 -02:00</code>.
     * @param cal input date
     * @param timeZone target timezone
     * @return a new calendar object in the target timezone and the same date and time fields as the original calendar
     *         object
     */
    public static Calendar replaceTimeZone(final Calendar cal, final TimeZone timeZone) {
		return null;
	}

    /**
     * Safely prints a {@link Calendar} object for debugging output. Shows only the important fields and is much more
     * readable than the standard {@link Calendar#toString()} output.
     */
    public static String calendarToString(final Calendar cal) {
		return null;
	}

    private static void pad(final StringBuffer buffer, final String str, final char padChr, final int len) {
	}

    /**
     * Calculates a year/month/day relative path based on the date.
     */
    public static String getDatePath(final java.util.Calendar date, final boolean isMonth) {
		return null;
	}

}
