/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.comments.api;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.lang.StringUtils;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.SyntheticResource;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.Attachment;
import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.commons.CommentUtil;
import com.adobe.cq.social.commons.client.api.BaseSocialComponent;
import com.adobe.cq.social.commons.client.api.ClientUtilities;
import com.adobe.cq.social.commons.client.api.CollectionPagination;
import com.adobe.cq.social.commons.client.api.CollectionSortedOrder;
import com.adobe.cq.social.commons.client.api.QueryRequestInfo;
import com.adobe.cq.social.commons.client.api.SocialComponent;
import com.adobe.cq.social.commons.client.api.SocialComponentFactory;
import com.adobe.cq.social.commons.client.api.SocialComponentFactoryManager;
import com.adobe.cq.social.commons.client.api.User;
import com.adobe.cq.social.commons.comments.listing.CommentIndexElementFactory;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentList;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProvider;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProviderManager;
import com.adobe.cq.social.commons.comments.listing.impl.CommentIndexElementFactoryImpl;
import com.adobe.cq.social.commons.moderation.impl.FlagReason;
import com.adobe.cq.social.commons.tagging.SocialTagManager;
import com.adobe.cq.social.tally.Response;
import com.adobe.cq.social.tally.TallyException;
import com.adobe.cq.social.tally.Vote;
import com.adobe.cq.social.tally.Voting;
import com.adobe.cq.social.translation.TranslationUtil;
import com.adobe.cq.social.ugcbase.CollabUser;
import com.adobe.granite.security.user.UserProperties;
import com.day.cq.tagging.TagConstants;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;

public abstract class AbstractComment<T extends CommentCollectionConfiguration> extends BaseSocialComponent implements
    Comment<T> {
    /** Logger for this class. */
    private static final Logger LOG = LoggerFactory.getLogger(AbstractComment.class);

    private static final String UNKNOWN_USER = "Unknown";

    /** local variables. */
    protected final CommentSocialComponentList replies;
    private final User author;
    private final Map<String, Attachment> attachments;
    private final Calendar created;
    private final String message;
    private final String parentPath;
    private final String resourceType;
    private final boolean isTopLevel;
    private final boolean isApproved;
    private boolean canEdit = false;
    private boolean canDelete = false;
    private boolean userIsModerator = false;
    private boolean currentUserFlagged = false;
    private String currentUserFlagText = null;
    private final QueryRequestInfo queryInfo;
    private final boolean isVisible;
    private final boolean isClosed;
    private final boolean doDisplayTranslation;
    private final boolean displayReplyButton;
    private T configuration;
    private final ModeratorActions moderatorActions;
    private final ModeratorStatus moderatorStatus;
    private final String votingRoot;
    private boolean useFlagReasonList = true;
    private boolean useFlagReasons = false;
    private List<FlagReason> flagReasons;
    private CommentSocialComponentListProviderManager commentListProviderManager;
    private final com.adobe.cq.social.commons.Comment comment;

    private PageInfo pageInfo;

    /**
     * Construct a comment for the specified resource and client utilities.
     * @param resource the specified resource
     * @param clientUtils the client utilities instance
     * @param commentListProviderManager list manager to use for listing content
     * @throws RepositoryException if an error occurs
     */
    public AbstractComment(final Resource resource, final ClientUtilities clientUtils,
        final CommentSocialComponentListProviderManager commentListProviderManager) throws RepositoryException {
        this(resource, clientUtils, QueryRequestInfo.DEFAULT_QUERY_INFO_FACTORY.create(), commentListProviderManager);
    }

    /**
     * Constructor of a comment.
     * @param resource the specified {@link com.adobe.cq.social.commons.Comment}
     * @param clientUtils the client utilities instance
     * @param queryInfo the query info.
     * @param commentListProviderManager list manager to use for listing content
     * @throws RepositoryException if an error occurs
     */
    public AbstractComment(final Resource resource, final ClientUtilities clientUtils,
        final QueryRequestInfo queryInfo, final CommentSocialComponentListProviderManager commentListProviderManager)
        throws RepositoryException {

        super(resource, clientUtils);
        this.commentListProviderManager = commentListProviderManager;
        comment = resource.adaptTo(com.adobe.cq.social.commons.Comment.class);
        author = this.clientUtils.getUser(comment.getAuthor().getId(), comment.getResource().getResourceResolver());
        attachments = comment.getAttachments();
        created = comment.getCreated();
        message = comment.getMessage();
        parentPath = comment.getComponent().getPath();
        isTopLevel = comment.isTopLevel();
        this.queryInfo = queryInfo;
        flagReasons = new ArrayList<FlagReason>();

        final Session session = resource.getResourceResolver().adaptTo(Session.class);
        final String userId = session.getUserID();

        final CommentSystem cs = comment.getCommentSystem();
        // Comment is approved or it doesn't need to be because it's not moderated.
        isApproved = comment.isApproved() || !cs.isModerated();
        userIsModerator = clientUtils.getSocialUtils().hasModeratePermissions(resource);

        final boolean userIsLoggedIn =
            clientUtils == null ? !(userId == null || userId.equalsIgnoreCase(CollabUser.ANONYMOUS)) : !clientUtils
                .userIsAnonymous();

        isClosed = comment.isClosed();
        configuration = createConfiguration(resource, cs);
        if ((userIsModerator || (userIsOwner() && userIsLoggedIn)) && !isClosed) {
            canEdit = true;
        }

        if ((userIsModerator || (userIsOwner() && userIsLoggedIn && cs.allowsDelete())) && !isClosed) {
            canDelete = true;
        }

        // A comment is visible to moderators, and to others if it's not spam, not hidden from flagging,
        // and not premoderated and not yet approved.
        if (userIsModerator || (!comment.isSpam() && !comment.isFlaggedHidden() && isApproved)) {
            isVisible = true;
        } else {
            isVisible = false;
        }
        displayReplyButton = cs.allowsReplies() && !isClosed;
        moderatorStatus = (userIsModerator) ? new ModeratorStatusImpl(comment, cs) : null;

        this.moderatorActions = new ModeratorActionsImpl(comment, cs, userIsLoggedIn);
        if (cs.allowsFlagging()) {
            useFlagReasonList = cs.useFlagReasonList();
            useFlagReasons = useFlagReasonList || cs.allowCustomFlagReason();
            final Response<Vote> response = getFlagResponseForUser(userId);
            if (response != null && response.getResponseValue() != null) { // this user DID flag this content
                currentUserFlagged = true;
                if (useFlagReasons) {
                    final Resource responseResource = response.getResource();
                    final ValueMap resourceProperties = responseResource.adaptTo(ValueMap.class);

                    // Should not be null, but ensure it's not anyways.
                    currentUserFlagText =
                        resourceProperties.get(com.adobe.cq.social.commons.Comment.PROP_FLAG_REASON, "");
                }
            }

            if (userIsModerator && useFlagReasons) {
                flagReasons = listFlagReasons(resource, clientUtils);
            }
        }

        resourceType = comment.getResource().getResourceType();
        if (queryInfo.isQuery()) {
            // We don't want to return the children of comments for a search
            // query because
            // there is no ListProvider for the old Social comments. Also, we
            // don't need this data for search.
            final List<Object> emptyList = Collections.emptyList();
            replies = CommentSocialComponentList.EMPTY_LIST;
        } else {
            final QueryRequestInfo repliesListInfo = QueryRequestInfo.DEFAULT_QUERY_INFO_FACTORY.create(queryInfo);
            final CollectionPagination currentPagination = repliesListInfo.getPagination();
            CollectionPagination repliesPagination;
            if (currentPagination.getEmbedLevel() != CollectionPagination.DEFAULT_EMBED_LEVEL) {
                // don't fetch nested replies
                repliesPagination =
                    new CollectionPagination(0, 0, currentPagination.getEmbedLevel() + 1,
                        currentPagination.getSelectedIndex(), this.configuration.getPageSize());
            } else {
                repliesPagination =
                    new CollectionPagination(currentPagination.getOffset(),
                        queryInfo.getPagination() == CollectionPagination.DEFAULT_PAGINATION ? this.configuration
                            .getPageSize() : currentPagination.getSize(),
                        currentPagination.getEmbedLevel() + 1, currentPagination.getSelectedIndex(),
                        this.configuration.getPageSize());
            }
            repliesListInfo.setPagination(repliesPagination);
            repliesListInfo.setSortOrder(CollectionSortedOrder.DEFAULT_ORDER);
            final CommentSocialComponentListProvider listProvider =
                this.commentListProviderManager.getCommentSocialComponentListProvider(resource, repliesListInfo);
            replies = listProvider.getCommentSocialComponentList(this, repliesListInfo, clientUtils);
            pageInfo = new PageInfo(this, clientUtils, repliesListInfo.getPagination());
        }

        // call the function from TranslationUtil to determine weather show the translation button.
        doDisplayTranslation = TranslationUtil.doDisplayTranslation(resource.getResourceResolver(), resource,
                clientUtils);

        final Resource component = cs.getResource();
        votingRoot = CommentUtil.getVotingRoot(component);

    }

    /**
     * Obtain the configuration for this comment.
     * @param resource
     * @param cs
     * @return
     */
    @Override
    public T createConfiguration(final Resource resource, final CommentSystem cs) {
		return null;
	}

    @Override
    public T getConfiguration() {
		return null;
	}

    private boolean userIsOwner() {
		return false;
	}

    /**
     * Get the path for the current version of the flag tally. This may or may not exist.
     * @return String containing the path, or null if the resource has no returnable properties.
     */
    private String getFlagsPath() {
		return null;
	}

    /**
     * Return response of current user to this comment. If this comment was not flagged by this user (or was last
     * unflagged), return null.
     * @return Response
     * @throws RepositoryException
     */
    private Response<Vote> getFlagResponseForUser(final String currentUserId) throws RepositoryException {
		return null;
	}

    /**
     * Get a list of flag reasons.
     * @param resource The comment resource
     * @param clientUtils
     * @return The list of flag reasons.
     * @throws RepositoryException
     */
    private List<FlagReason> listFlagReasons(final Resource resource, final ClientUtilities clientUtils)
        throws RepositoryException {
			return null;
		}

    @Override
    protected List<String> getIgnoredProperties() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Attachment getAttachment(final String name) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public User getAuthor() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, Attachment> getAttachments() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Calendar getCreated() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getResourceType() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getMessage() {
		return null;
	}

    /**
     * Gets the parent path for the comment.
     * @return a string pointing to the externalized URL to the parent.
     */
    public String getParent() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
		return null;
	}

    /**
     * Set the collection list range. If this is not set, then the return list will start at offset 0 and the default
     * size
     * @param pagination detail information to use
     */
    @Override
    public void setPagination(final CollectionPagination pagination) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void setSortedOrder(final CollectionSortedOrder sortedOrder) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isTopLevel() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isApproved() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanEdit() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanReply() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanDelete() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public int getTotalSize() {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Object> getItems() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getParentId() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanTranslate() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isUserModerator() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isVisible() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isClosed() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public List<FlagReason> getFlagReasons() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getCurrentUserFlagText() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getUseFlagReasons() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isFlaggedByUser() {
		return false;
	}

    @Override
    public SocialComponent getVotes() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public ModeratorActions getModeratorActions() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    @JsonInclude(Include.NON_NULL)
    public ModeratorStatus getModeratorStatus() {
		return null;
	}

    /**
     * Get the indexer to use based on the current QueryRequestInfo.
     * @return element factory that will track the index
     */
    @SuppressWarnings("rawtypes")
    protected CommentIndexElementFactory getIndexElementFactory() {
		return null;
	}

    /**
     * Get the list of replies.
     * @return the children of this comment.
     * @throws RepositoryException is thrown if there an error occurs while fetching the data.
     */
    protected List<Comment> getComments() throws RepositoryException {
		return null;
	}

    /**
     * Get the pagination information for the comment.
     * @return the QueryRequestInfo pagination details
     */
    protected CollectionPagination getPagination() {
		return null;
	}

    /**
     * Get the query info comment.
     * @return the QueryRequestInfo details
     */
    protected QueryRequestInfo getQueryRequestInfo() {
		return null;
	}

    @Override
    public List<Tag> getTags() {
		return null;
	}

    protected class ModeratorStatusImpl implements ModeratorStatus {
        private final boolean flagged, approved, spam;

        public ModeratorStatusImpl(final com.adobe.cq.social.commons.Comment comment, final CommentSystem cs) {
            flagged = comment.isFlagged();
            approved = comment.isApproved() || !cs.isModerated();
            spam = comment.isSpam();
        }

        @Override
        @JsonProperty("isFlagged")
        public boolean isFlagged() {
			return false;
		}

        @Override
        @JsonProperty("isApproved")
        public boolean isApproved() {
			return false;
		}

        @Override
        @JsonProperty("isSpam")
        public boolean isSpam() {
			return false;
		}
    }

    protected class ModeratorActionsImpl implements ModeratorActions {
        private boolean displayAllowButton = false;
        private boolean displayFlagButton = false;
        private boolean displayDenyButton = false;
        private boolean displayCloseButton = false;

        public ModeratorActionsImpl(final com.adobe.cq.social.commons.Comment comment, final CommentSystem cs,
            final boolean userIsLoggedIn) {
            final boolean commentApproved = comment.isApproved() || !cs.isModerated();

            // Logic explanation:
            // HIDE the Flag button if any of these are true
            // The user is not logged in
            // The comment is not closed
            // The user owns the the resource (or is a mod, as they are considered the owner ??)
            // The comment is spam
            // The comment has been flagged enough times to be hidden
            // The the comment is flagged for moderation and the user is a moderator
            // The comment is not approved (if a premoderated system)
            // The comment system doesn't allow flagging.
            displayFlagButton =
                userIsLoggedIn && !isClosed && !userIsOwner() && !comment.isSpam() && !comment.isFlaggedHidden()
                        && !comment.isFlagged() && commentApproved && cs.allowsFlagging();

            displayAllowButton =
                userIsModerator && !isClosed && (comment.isSpam() || comment.isFlagged() || !commentApproved);

            displayDenyButton = userIsModerator && !isClosed && !comment.isSpam() && cs.allowsDeny();

            // close or open button are both covered under displayCloseButton:
            displayCloseButton = userIsModerator && isTopLevel && cs.allowsClose();
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanDeny() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getUseFlagReasonList() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanAllow() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanFlag() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanClose() {
			return false;
		}
    }

    /**
     * Gets information about the pages for this collection. This can be used by the page block helper to easily
     * render various pagination UIs.
     * @return information about the pagination system
     */
    @Override
    public PageInfo getPageInfo() {
		return null;
	}
    
    /**
     * @return returns the reference to {@link com.adobe.cq.social.commons.Comment} object represented by this
     *         instance.
     */
    protected com.adobe.cq.social.commons.Comment getComment() {
		return null;
	}
}
