/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.security;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Properties;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.request.RequestParameter;
import org.osgi.service.component.ComponentContext;

import com.adobe.cq.social.commons.AttachmentTypeBlacklistService;
import com.adobe.cq.social.commons.SaferSlingPostValidator;
import com.adobe.cq.social.ugcbase.AttachmentDataSource;
import com.day.text.Text;

/**
 * The <code>SaferSlingPostValidatorImpl</code> checks Sling POST requests for safety and recommends to REJECT or
 * ACCEPT based on safe/unsafe constructs in the request.
 */
@Component(label = "AEM Social Communities SaferSlingPostValidator", metatype = true)
@Service(serviceFactory = true, value = SaferSlingPostValidator.class)
@Properties({@Property(name = "service.description", value = "Safer Sling Post Validator", propertyPrivate = true)})
public class SaferSlingPostValidatorImpl implements SaferSlingPostValidator {

    private static final String OPERATION_PARAMETER = ":operation";
    private static final String TYPEHINT_SUFFIX = "@TypeHint";
    private static final String RESOURCETYPE_PARAMETER = "sling:resourceType";
    private static final String RESOURCESUPERTYPE_PARAMETER = "sling:resourceSuperType";
    private static final String JCR_PRIMARYTYPE = "jcr:primaryType";
    private static final String JCR_MIXINTYPES = "jcr:mixinTypes";
    private static final String BINARY = "Binary";

    /**
     * Properties with : in them are often security significant, by default they are not allowed through, with certain
     * 'safe' values white listed. Properties starting with : in them are automatically white listed.
     */
    @Property(value = {"jcr:description", "jcr:title", "jcr:lastModified", JCR_PRIMARYTYPE, JCR_MIXINTYPES,
        RESOURCETYPE_PARAMETER, RESOURCESUPERTYPE_PARAMETER, "cq:tags"})
    private static final String PARAMETER_WHITELIST = "parameter.whitelist";

    /**
     * This configuration allows white listing certain parameter name prefixes such as social: which are known to be
     * safe.
     */
    @Property(value = {"social:"}, cardinality = Integer.MAX_VALUE)
    private static final String PARAMETER_WHITELIST_PREFIXES = "parameter.whitelist.prefixes";

    /**
     * Binary properties with . in them can trigger mime type guessing from the 'extension' part when served by the
     * DefaultGetServlet. Therefore such properties are excluded by default, with a white list which is empty by
     * default.
     */
    @Property(value = {}, cardinality = Integer.MAX_VALUE)
    private static final String BINARY_PARAMETER_WHITELIST = "binary.parameter.whitelist";

    /**
     * The SlingPostServlet supports some powerful @ suffixes for parameter names to control copying from existing
     * content for example. But some of these suffixes allow bypassing security otherwise provided by the Dispatcher.
     * So only white listed suffixes are allowed.
     */
    @Property(value = {TYPEHINT_SUFFIX, "@DefaultValue", "@UseDefaultWhenMissing", "@IgnoreBlanks", "@ValueFrom",
        "@Delete", "@Patch"})
    private static final String MODIFIER_WHITELIST = "modifier.whitelist";

    /**
     * The SlingPostServlet supports some powerful :operation values such as import/export. So only the basic safe
     * operations are allowed.
     */
    @Property(value = {"delete", "nop"})
    private static final String OPERATION_WHITELIST = "operation.whitelist";

    /**
     * The SlingPostServlet supports some powerful :operation values such as import/export. So only specific
     * operations or operations implemented to be safe and given a specific name prefix are allowed.
     */
    @Property(value = {"social:"}, cardinality = Integer.MAX_VALUE)
    private static final String OPERATION_WHITELIST_PREFIXES = "operation.whitelist.prefixes";

    /**
     * The TypeHint suffix does double duty: selecting property types and node types. Node types are significant
     * because the JCR/Sling observers often filter based on node type, and can place undue trust in the content of
     * nodes bearing the expected node type. Further node types set default sling:ResourceTypes which are used to
     * locate servlets. Allowing attackers to choose unexpected servlets increases the attack surface area
     * drastically. We allow all known property types, but a 'safe' subset of node types.
     */
    @Property(value = {"cq:CalendarEvent", "nt:unstructured", "nt:folder", "nt:file", "nt:resource", "sling:Folder",
        "sling:OrderedFolder", BINARY, "Boolean", "Date", "Double", "Long", "Name", "Path", "String"})
    private static final String TYPEHINT_WHITELIST = "typehint.whitelist";

    /**
     * Resource types are the means by which Sling selects which servlets to use to service requests. Allowing an
     * attacker full range to choose any existing servlet would drastically increase the attack surface, so by default
     * all resource type settings (sling:resourceType and sling:resourceSuperType) are rejected, except those known to
     * be needed by features. Candidates on the whitelist should be strongly vetted for XSS, CSRF and any other
     * security vulnerabilities.
     */
    @Property(value = {"social/calendar/components/event"}, cardinality = Integer.MAX_VALUE)
    private static final String RESOURCETYPE_WHITELIST = "resourcetype.whitelist";

    @Reference
    private AttachmentTypeBlacklistService attachmentTypeBlacklist;

    private Set<String> parameterWhiteList = new HashSet<String>();
    private Set<String> parameterWhiteListPrefixes = new HashSet<String>();
    private Set<String> binaryParameterWhiteList = new HashSet<String>();
    private Set<String> modifierWhiteList = new HashSet<String>();
    private Set<String> operationWhiteList = new HashSet<String>();
    private Set<String> operationWhiteListPrefixes = new HashSet<String>();
    private Set<String> typeHintWhiteList = new HashSet<String>();
    private Set<String> resourceTypeWhiteList = new HashSet<String>();

    /**
     * Grab all our configurations.
     * @param ctx our component context.
     */
    @Activate
    protected void activate(final ComponentContext ctx) {
	}

    @Override
    public boolean reject(final SlingHttpServletRequest request) {
		return false;
	}

    private boolean checkParameterPrefixes(final String name) {
		return false;
	}

    private String leafName(final String name) {
		return null;
	}

    private boolean checkResourceTypes(final SlingHttpServletRequest request, final String name, final String fullName) {
		return false;
	}

    private boolean checkJCRTypes(final SlingHttpServletRequest request, final String name, final String fullName) {
		return false;
	}

    private boolean checkOperation(final SlingHttpServletRequest request, final String name, final String fullName) {
		return false;
	}

    private boolean checkModifier(final SlingHttpServletRequest request, final String name, final String fullName,
        final String modifier) {
			return false;
		}

    private boolean checkTypeHint(final SlingHttpServletRequest request, final String name, final String fullName,
        final String modifier) {
			return false;
		}

    private boolean validateApplyTo(final SlingHttpServletRequest request) {
		return false;
	}

    private boolean validateApplyToPath(final String applyToPath, final String resourcePath, final int depth) {
		return false;
	}

    private boolean validateUploads(final SlingHttpServletRequest request, final String name) {
		return false;
	}
}
