/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.sc.lucene;

import static com.adobe.cq.social.sc.lucene.TransformConstants.TRANSFORM_LOWER_CASE;
import static com.adobe.cq.social.sc.lucene.TransformConstants.TRANSFORM_NONE;
import static com.adobe.cq.social.sc.lucene.TransformConstants.TRANSFORM_UPPER_CASE;
import static javax.jcr.PropertyType.DATE;
import static javax.jcr.PropertyType.DECIMAL;
import static javax.jcr.PropertyType.DOUBLE;
import static javax.jcr.PropertyType.LONG;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_EQUAL_TO;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_GREATER_THAN;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_GREATER_THAN_OR_EQUAL_TO;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_LESS_THAN;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_LESS_THAN_OR_EQUAL_TO;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_LIKE;
import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_OPERATOR_NOT_EQUAL_TO;
import static org.apache.lucene.search.BooleanClause.Occur.MUST;
import static org.apache.lucene.search.BooleanClause.Occur.MUST_NOT;
import static org.apache.lucene.search.BooleanClause.Occur.SHOULD;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.query.InvalidQueryException;
import javax.jcr.query.Row;
import javax.jcr.query.qom.And;
import javax.jcr.query.qom.ChildNode;
import javax.jcr.query.qom.Comparison;
import javax.jcr.query.qom.Constraint;
import javax.jcr.query.qom.DescendantNode;
import javax.jcr.query.qom.DynamicOperand;
import javax.jcr.query.qom.FullTextSearch;
import javax.jcr.query.qom.FullTextSearchScore;
import javax.jcr.query.qom.Length;
import javax.jcr.query.qom.LowerCase;
import javax.jcr.query.qom.NodeLocalName;
import javax.jcr.query.qom.NodeName;
import javax.jcr.query.qom.Not;
import javax.jcr.query.qom.Or;
import javax.jcr.query.qom.Ordering;
import javax.jcr.query.qom.PropertyExistence;
import javax.jcr.query.qom.PropertyValue;
import javax.jcr.query.qom.QueryObjectModelConstants;
import javax.jcr.query.qom.SameNode;
import javax.jcr.query.qom.Selector;
import javax.jcr.query.qom.StaticOperand;
import javax.jcr.query.qom.UpperCase;

import org.apache.jackrabbit.commons.predicate.Predicate;
import org.apache.jackrabbit.commons.predicate.Predicates;
import org.apache.jackrabbit.commons.predicate.RowPredicate;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.PrefixQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.SortField;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.TermRangeQuery;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.WildcardQuery;
import org.apache.lucene.store.Directory;
import org.apache.lucene.util.Version;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.ugc.api.SearchResults;
import com.adobe.cq.social.ugc.impl.ListSearchResults;
import com.day.crx.sc.qom.OperandEvaluator;
import com.day.crx.sc.qom.SelectorRow;
import com.day.crx.sc.qom.ValueComparator;

/**
 * Factory that creates Lucene queries from QOM elements.
 */
public class LuceneQueryFactory {

    private static final boolean ASCENDING = false;
    private static final boolean DESCENDING = true;

    /**
     * The logger.
     */
    private static final Logger LOG = LoggerFactory.getLogger(LuceneQueryFactory.class);

    /** Lucene directory. */
    private final Directory directory;

    /** Lucene analyzer. */
    private final Analyzer analyzer;

    /**
     * Session of the user executing this query.
     */
    private final Session session;

    /**
     * Node type manager
     */
    private final NodeTypeManager ntManager;

    /** Operand evaluator. */
    private final OperandEvaluator evaluator;

    /**
     * Creates a new lucene query factory.
     * @param session the session that executes the query.
     * @param bindVariables the bind variable values of the query
     */
    public LuceneQueryFactory(final Directory directory, final Analyzer analyzer, final Session session,
        final Map<String, Value> bindVariables) throws RepositoryException {
        this.directory = directory;
        this.analyzer = analyzer;
        this.session = session;
        this.ntManager = session.getWorkspace().getNodeTypeManager();
        this.evaluator = new OperandEvaluator(session.getValueFactory(), bindVariables);
    }

    public SearchResults<Row> execute(final Map<String, PropertyValue> columns, final Selector selector,
        final Constraint constraint, final Ordering[] orderings, final long offset, final long maxResults)
        throws RepositoryException, IOException {
			return null;
		}

    /**
     * Get the lucene query statement.
     * @param columns columns
     * @param selector the selector
     * @param constraint the constraint
     * @param orderings sort
     * @return the query statement
     * @throws RepositoryException on repository error
     * @throws IOException on I/O error
     */
    public String getStatement(final Map<String, PropertyValue> columns, final Selector selector,
        final Constraint constraint, final Ordering[] orderings) throws RepositoryException, IOException {
			return null;
		}

    /**
     * Creates a lucene query for the given QOM selector.
     * @param selector the selector.
     * @return a lucene query for the given selector.
     * @throws RepositoryException if an error occurs while creating the query.
     */
    public Query create(final Selector selector) throws RepositoryException {
		return null;
	}

    protected Term createNodeTypeTerm(final NodeType type) throws RepositoryException {
		return null;
	}

    /**
     * Creates a lucene query for the given QOM full text search.
     * @param fts the full text search constraint.
     * @return the lucene query for the given constraint.
     * @throws RepositoryException if an error occurs while creating the query.
     */
    public Query create(final FullTextSearch fts) throws RepositoryException {
		return null;
	}

    /**
     * Creates a lucene query for the given QOM property existence constraint.
     * @param prop the QOM constraint.
     * @return the lucene query for the given constraint.
     * @throws RepositoryException if an error occurs while creating the query.
     */
    public Query create(final PropertyExistence prop) throws RepositoryException {
		return null;
	}

    protected Predicate mapConstraintToQueryAndFilter(final BooleanQuery query, final Constraint constraint,
        final Map<String, NodeType> selectorMap) throws RepositoryException, IOException {
			return null;
		}

    protected Query create(final Constraint constraint, final Map<String, NodeType> selectorMap)
        throws RepositoryException, IOException {
			return null;
		}

    protected BooleanQuery getAndQuery(final And and, final Map<String, NodeType> selectorMap)
        throws RepositoryException, IOException {
			return null;
		}

    protected BooleanQuery getOrQuery(final Or or, final Map<String, NodeType> selectorMap)
        throws RepositoryException, IOException {
			return null;
		}

    protected BooleanQuery getNotQuery(final Not not, final Map<String, NodeType> selectorMap)
        throws RepositoryException, IOException {
			return null;
		}

    protected void addBooleanConstraint(final BooleanQuery query, final Constraint constraint, final Occur occur,
        final Map<String, NodeType> selectorMap) throws RepositoryException, IOException {
		}

    protected static class Transform {

        private final DynamicOperand operand;

        private final int transform;

        public Transform(DynamicOperand operand) {
            // Check the transformation type
            if (operand instanceof UpperCase) {
                this.transform = TRANSFORM_UPPER_CASE;
            } else if (operand instanceof LowerCase) {
                this.transform = TRANSFORM_LOWER_CASE;
            } else {
                this.transform = TRANSFORM_NONE;
            }

            // Unwrap any nested transformations
            while (true) {
                if (operand instanceof UpperCase) {
                    operand = ((UpperCase) operand).getOperand();
                } else if (operand instanceof LowerCase) {
                    operand = ((LowerCase) operand).getOperand();
                } else {
                    break;
                }
            }
            this.operand = operand;
        }
    }

    protected Query getComparisonQuery(final DynamicOperand left, final int transform, final String operator,
        final StaticOperand rigth, final Map<String, NodeType> selectorMap) throws RepositoryException {
			return null;
		}

    protected Query getNodeNameQuery(final int transform, final String operator, final StaticOperand right)
        throws RepositoryException {
			return null;
		}

    protected Query getNodeLocalNameQuery(final int transform, final String operator, final StaticOperand right)
        throws RepositoryException {
			return null;
		}

    protected Query getPropertyValueQuery(final String field, final String operator, final Value value,
        final int type, final int transform) throws RepositoryException {
			return null;
		}

    private String getValueString(final Value value, final int type) throws RepositoryException {
		return null;
	}

}
