/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.translation.api;

import java.util.Map;

/**
 * The TranslationService interface provides a single source for all Machine Translation (MT) functionality.
 *
 * <p>
 * This interface (1.0) provides functions for Translation, Detection, and Translation Memory (TM) Updates.
 * This interface is implemented by Translation Connectors to provide connectivity to external MT engines.  The
 * Connectors contain the engine specific logic to authenticate against and call their respective MT engines.
 * </p>
 *
 * <p>
 * The service is implemented in three parts.
 * </p>
 *
 * <p>
 *
 * <b>Helper methods</b> that make Machine Translation Possible
 * <ul>
 *  <li><b>isServiceAvailable:</b> check if a Machine Translation Service is connected and available for use.
 *  <li><b>isDirectionSupported:</b> check if a Machine Translation Service can translate between a particular source and
 * target pair in that direction.
 *  <li><b>detectLanguage:</b> determine the language identity (French, German, etc.) of a source string.
 * </ul>
 * </p>
 *
 * <p>
 * <b>Translation</b> methods that do the actual translation
 * <ul>
 * <li> <b>translateString:</b> translate a single string from source language into target language
 * <li> <b>translateArray:</b> translate an array of strings from one source language into one target language
 * </ul>
 * </p>
 *
 * <p>
 * <b>Storage and Retrieval</b> methods that store translation with an MT provider and retrieve those translations.
 * <ul>
 * <li><b>getStoredTranslations:</b> send a source string for translation and retrieve existing ranked translations for that
 * source string.
 * <li><b>addStoredTranslation:</b> push an target translation for a particular source string into a services storage repository. (string and array versions)
 * </ul>
 * </p>
 *
 */
public interface TranslationService {

    /**
     * Returns the full list of supported language codes for this service.  This is a comprehensive
     * list of all supported languages.  Use supportedLanguages to determine which languages are supported by a
     * TranslationService. It returns a human readable name for the languages as well a language code for those
     * languages. The service has the capability to recognize these codes and the language named. These are the
     * languages which detectLanguage can recognize. It doesn't not necessarily mean the service can translate
     * between all these languages. Use isDirectionSupported (see below) to retrieve the subset of direction pairs.
     *
     * @return @code{Map<String, String>} containing the full list of supported languages and their English names.
     */
    Map<String, String> supportedLanguages() {
		return null;
	}

    /**
     * Returns true/false whether the current service supports the specified translation direction.  At it's
     * simplest a translation service consists of a language pair and a direction. A language pair is any two
     * languages. For example: French and English. A direction is which language is the Source language and which
     * language is the Target language. For example: I want to translate from English to French,
     * so English would be the Source and French would be the Target. If the service supports translation from
     * English to French that would be one direction that was supported. Similarly if the service supports
     * translation from French to English that would be another direction. Use isDirectionSupported to learn if the
     * required language pair and direction is available.
     *
     * @param sourceLanguage The language code to be translated from
     * @param targetLanguage The language code to be translated to
     * @return Returns true if engine supports direction, false otherwise
     */
    boolean isDirectionSupported(String sourceLanguage, String targetLanguage)
            throws TranslationException {
		return false;
	}

    /**
     * Detects the language of the passed in String (accepts both PLAIN and HTML text).  Use to determine the
     * language of piece of content. For example: A forum user comments on a discussion thread in their native
     * script. The forum administrator would like to provide a translation of this comment for all the users,
     * but doesn't recognize the language it was posted in. The method detectLanguage makes known the language ID
     * of the comment in question so it may be translated.
     *
     * @param detectSource The content to determine the language of
     * @param contentType The content type of the source (@see TranslationConstants.ContentTypes)
     * @return Returns the language code of the passed in string (language or locale (if known))
     *         Returns "UNKNOWN" if language cannot be determined (@see TranslationException
     *         .ErrorCodes#UNKNOWN_LANGUAGE)
     */
    String detectLanguage(String detectSource, TranslationConstants.ContentType contentType)
            throws TranslationException {
		return null;
	}

    /**
     * Translates a given String and returns the best fit (highest rated) translation.  The connector may determine
     * a specific engine to use based on the input parameters.  This is the core of the translation API. A user
     * calls the translateString method to send a Source string for translation. The translation (the Target
     * String) is then returned to the user.
     *
     * @param sourceString      The string to be translated
     * @param sourceLanguage    (optional) The string representing the FROM language code.  This allows a NULL
     *                          value to be passed.  If NULL, the source language is determined by leveraging the
     *                          "detectLanguage" function.  A TranslationException with ErrorCode UNKNOWN_LANGUAGE
     *                          will be thrown if it cannot be determined.
     * @param targetLanguage    The string representing the TO language code.
     * @param contentType       The content type of the source (@see TranslationConstants.ContentTypes)
     * @param contentCategory   (optional) The content category of the source (@see TranslationConstants.Categories)
     *                          This will use the default value if NULL or blank is passed.
     * @return                  A translation of the "sourceString" String in the "targetLanguage" (best match)
     */
    TranslationResult translateString(String sourceString, String sourceLanguage, String targetLanguage,
                                      TranslationConstants.ContentType contentType, String contentCategory)
            throws TranslationException {
		return null;
	}

    /**
     * Translates all strings in a given String Array and returns the best fit (highest rated) translations.  The
     * connector may determine a specific engine to use based on the input parameters.  This function only allows
     * one direction (TO/FROM pair).  This method is like translateString with one difference. The translateArray
     * method sends an Array of strings out for translation at the same time. Translations are returned for each
     * string in the array. This allows the user to batch process strings for translation.
     *
     * @param sourceStringArr      An Array of Strings to be translated
     * @param sourceLanguage       (optional) The string representing the FROM language code.
     *                             This allows a NULL value to be passed.  If NULL, the source language is determined
     *                             by leveraging the "detectLanguage" function.  A TranslationException with
     *                             ErrorCode UNKNOWN_LANGUAGE will be thrown if it cannot be determined.
     *                             NOTE:  Only the first array position will be used to determine a NULL or blank FROM value.
     * @param targetLanguage       The string representing the TO language code.
     * @param contentType          The content type of the source (@see TranslationConstants.ContentTypes)
     * @param contentCategory      (optional) The content category of the source (@see TranslationConstants.Categories)
     *                             This will use the default value if NULL or blank is passed.
     * @return An array of TranslationReturnObjects in the same order as the sourceStringArr
     */
    TranslationResult[] translateArray(String[] sourceStringArr, String sourceLanguage, String targetLanguage,
                                       TranslationConstants.ContentType contentType, String contentCategory)
            throws TranslationException {
		return null;
	}

    /**
     * Returns all available translation responses for a given string, array will be sorted by rating (best fit
     * first, in "0").  This function is only available if this connector leverages a Translation Memory system.
     * Use getAllStoredTranslations when the user would like to retrieve not only the highest ranked Machine
     * Translated options but also any other alternate translations as they are ranked. These stored Translation
     * Memory options are rated by the admin/moderator. This permits service admin/moderator to better control
     * the translations returned by the Machine Translation Service. It also may be used to support the moderation
     * of crowd sourced and edited translations.
     *
     * @param sourceString      The string to be translated
     * @param sourceLanguage    (optional) The string representing the FROM language code.
     *                          This allows a NULL value to be passed.  If NULL, the source language is determined
     *                          by leveraging the "detectLanguage" function. A TranslationException with
     *                          ErrorCode UNKNOWN_LANGUAGE will be thrown if it cannot be determined.
     * @param targetLanguage    The string representing the TO language code.
     * @param contentType       The content type of the source (@see TranslationConstants.ContentTypes)
     * @param contentCategory   (optional) The content category of the source (@see TranslationConstants.Categories)
     *                          This will use the default value if NULL or blank is passed.
     * @param userId            (optional) filters the results specific to a passed in User
     * @param maxTranslations   (optional) Specifies the maximum number of translation response Strings in array.
     *                          If not specified ALL available translations will be returned.
     * @return                  An array of TranslationReturnObjects containing all translations within the Machine
     *                          translation store
     */
    TranslationResult[] getAllStoredTranslations(String sourceString, String sourceLanguage, String targetLanguage,
                                                 TranslationConstants.ContentType contentType, String contentCategory,
                                                 String userId, int maxTranslations)
            throws TranslationException {
		return null;
	}

    /**
     * Updates a translation in the Translation Memory engine.  Use storeTranslation when an
     * admin/moderator would like to push a reviewed translation into Translation Memory for future use. This
     * API is used for overwriting to the MT service. Stored Translations are added to Translation Memory with
     * a confidence rating. If there is an exact match with a stored translation the translation with the
     * highest confidence level rating will be returned. The default score for translation quality in the TIF
     * (Translation Integration Framework) is fixed at "0" on a scale from "-n" to "+n". To work with the
     * different scales of external services "-n" and "+n" can represent any value range (i.e. -100 .. +100).
     * Individual connectors for each translation service (synchronous or asynchronous) handle conversions
     * between this scale and the scale used by the service being called.
     *
     * @param originalText       The string that was originally translated.  This can be only one sentence.
     * @param sourceLanguage       The string representing the locale that was translated from (source)
     *                           This accepts all acceptable locale strings (2 and 5 char versions)
     *                           Example:  en, en_us, en-us, en_US
     * @param targetLanguage         The string representing the locale that was translated to (destination)
     *                           This accepts all acceptable locale strings (2 and 5 char versions)
     *                           Example:  en, en_us, en-us, en_US
     * @param updatedTranslation The string representing the updated text after an editor reviewed and modified
     * @param contentType        @see TranslationConstants.ContentTypes
     * @param contentCategory    (optional) The Content Category @see TranslationConstants.Categories
     * @param userId             The user who modified the translation
     * @param rating             (optional) A number representing the "quality" rating of the new translation.
     *                           The scale is from -n .. +n with 0 being the default setting for translation
     *                           quality. 
     * @param path               (optional) Path to the node containing the translated content
     */
    void storeTranslation(String originalText, String sourceLanguage, String targetLanguage,
                          String updatedTranslation,
                          TranslationConstants.ContentType contentType, String contentCategory, String userId,
                          int rating, String path)
            throws TranslationException {
	}

    /**
     * Updates an array of translations in the Translation Memory engine.  Use storeTranslation when an
     * admin/moderator would like to push a reviewed translation into Translation Memory for future use. This API
     * is used for overwriting to the MT service. Stored Translations are added to Translation Memory with a
     * confidence rating. If there is an exact match with a stored translation the translation with the highest
     * confidence level rating will be returned. The default score for translation quality in the TIF
     * (Translation Integration Framework) is fixed at "0" on a scale from "-n" to "+n". To work with the
     * different scales of external services "-n" and "+n" can represent any value range (i.e. -100 .. +100).
     * Individual connectors for each translation service (synchronous or asynchronous) handle conversions
     * between this scale and the scale used by the service being called.
     *
     * @param originalText       An array of the original text that was translated
     * @param sourceLanguage       The string representing the locale that was translated from (source)
     *                           This accepts all acceptable locale strings (2 and 5 char versions)
     *                           Example:  en, en_us, en-us, en_US
     * @param targetLanguage         The string representing the locale that was translated to (destination)
     *                           This accepts all acceptable locale strings (2 and 5 char versions)
     *                           Example:  en, en_us, en-us, en_US
     * @param updatedTranslation An array of strings containing the updated translations,
     *                           this must be in the same order as the originalText array (originalText[1] is the
     *                           original text of translated string updatedTranslation[1]).  Thus originalText
     *                           .length must == updatedTranslation.length or an exception will be thrown.
     * @param contentType        @see TranslationConstants.ContentTypes
     * @param contentCategory    (optional) The Content Category @see TranslationConstants.Categories
     * @param userId             The user who modified the translation
     * @param rating             (optional) A number representing the "quality" rating of the new translation.
     *                           The scale is from -n .. +n with 0 being the default setting for translation
     *                           quality. 
     * @param path               (optional) Path to the node containing the translated content
     */
    void storeTranslation(String[] originalText, String sourceLanguage, String targetLanguage,
                          String[] updatedTranslation, TranslationConstants.ContentType contentType,
                          String contentCategory, String userId, int rating, String path)
            throws TranslationException {
	}

    /**
     * @return Returns a the configured Default Category for this TranslationService instance
     */
    String getDefaultCategory() {
		return null;
	}

    /**
     * @param defaultCategory The string representing the category to set as the default.  The TranslationService
     *                        will use this value if no category is specified.
     */
    void setDefaultCategory(String defaultCategory) {
	}

    /**
     * @return Returns a TranslationInfo object which contains the Name, Label, and Attribution Strings for this
     * specific TranslationService implementation
     */
    TranslationServiceInfo getTranslationServiceInfo() {
		return null;
	}

    public interface TranslationServiceInfo{

        /**
         * @return Optional property used to attribute this translation provider (such as "Translations by Adobe Translate")
         */
        String getTranslationServiceAttribution() {
			return null;
		}

        /**
         * @return Optional property used to describe this translation service in a user-interface (such as to allow
         * its selection from a drop-down menu).
         */
        String getTranslationServiceLabel() {
			return null;
		}

        /**
         * @return Name of the service property indicating the particular implementation this factory provides,
         * e.g. adobe.
         */
        String getTranslationServiceName() {
			return null;
		}
    }

}