/*
 * Copyright 1997-2010 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.commons.mail;

import com.day.cq.commons.jcr.JcrConstants;
import org.apache.commons.io.IOUtils;
import org.apache.commons.io.input.CountingInputStream;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.text.StrLookup;
import org.apache.commons.lang.text.StrSubstitutor;
import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.internet.InternetHeaders;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;

/**
 * The <code>MessageTemplate</code> class provides email text templating functionality. Templates are
 * <code>nt:file</code> nodes in the repository representing a text file.
 * <p/>
 * The text file contains the complete email. Email headers are defined as the first lines of the file in the format
 * <code>Header-Name: Header-Value</code>, one header per line. Headers supporting multiple values can thus have several
 * header lines. The supported headers are the standard email headers.
 * <p/>
 * After the last header line put an empty line and start the email body afterwards.
 * <p/>
 * Within all of the text file, replacement variables can be used in the form of <code>${variable}</code>, e.g.
 * <code>${payload.title}</code>. The available variables are defined by the variable resolver given in the {@link
 * #getEmail(org.apache.commons.lang.text.StrLookup, Class)} method.
 * <p/>
 * The <code>getEmail</code> method returns the chosen (<code>type</code> argument) email implementation, as long as the
 * type extends {@link org.apache.commons.mail.Email} and has a publically accessible default constructor. Out of the
 * box the three email implementations provided by the <i>Apache Commons Email</code> library can be used: {@link
 * org.apache.commons.mail.SimpleEmail}, {@link org.apache.commons.mail.HtmlEmail} and {@link
 * org.apache.commons.mail.MultiPartEmail}.
 */
public class MailTemplate {

    private static final String HEADER_TO = "To";
    private static final String HEADER_CC = "CC";
    private static final String HEADER_BCC = "BCC";
    private static final String HEADER_REPLYTO = "Reply-To";
    private static final String HEADER_FROM = "From";
    private static final String HEADER_SUBJECT = "Subject";
    private static final String HEADER_BOUNCETO = "Bounce-To";

    private static final String[] PRIMARY_HEADERS = new String[]{
            HEADER_TO,
            HEADER_CC,
            HEADER_BCC,
            HEADER_REPLYTO,
            HEADER_FROM,
            HEADER_SUBJECT,
            HEADER_BOUNCETO
    };

    private static final Logger log = LoggerFactory.getLogger(MailTemplate.class);

    private String message;

    /**
     * Constructs a new <code>MailTemplate</code> with the template text given as the <code>inputStream</code> and the
     * encoding (may be null).
     *
     * @param inputStream The template text.
     * @param encoding    The encoding of the input stream (may be null). If empty, <i>UTF-8</i> will be used.
     *
     * @throws IOException If an error occurs handling the input stream.
     */
    public MailTemplate(final InputStream inputStream, final String encoding) throws IOException {

        if (null == inputStream) {
            throw new IllegalArgumentException("input stream may not be null");
        }

        final InputStreamReader reader =
                new InputStreamReader(inputStream, StringUtils.defaultIfEmpty(encoding, "utf-8"));
        final StringWriter writer = new StringWriter();
        IOUtils.copy(reader, writer);
        message = writer.toString();
    }

    /**
     * Create an {@link org.apache.commons.mail.Email} based on the template text and replacing variables in the
     * template text using the given <code>lookup</code> implementation. Emails are constructed and returned with the
     * given <code>type</code>.
     *
     * @param lookup The {@link org.apache.commons.lang.text.StrLookup} implementation to use for variable lookup.
     * @param type   The class defining the email type.
     * @param <T>    The email type.
     *
     * @return An email based on the template text and the variable resolver.
     *
     * @throws IOException        If an error occurs handling the text template.
     * @throws MessagingException If an error occurs during building the email message.
     * @throws EmailException     If an error occurs during building the email.
     */
    public <T extends Email> T getEmail(final StrLookup lookup, final Class<T> type)
            throws IOException, MessagingException, EmailException {
				return null;
			}

    /**
     * Convenience method to create a new {@link com.day.cq.commons.mail.MailTemplate} based on the <code>path</code>
     * identifying the location of the email template text in the repository.
     *
     * @param path    The location of the email template text in the repository. Must point to an <i>nt:file</i> node.
     * @param session The session used for accessing the repository.
     *
     * @return A mail template or <code>null</code> if there was an error creating the template.
     */
    public static MailTemplate create(final String path, final Session session) {
		return null;
	}
}
