/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.commons.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.PushbackInputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;
import javax.jcr.query.Row;
import javax.jcr.query.RowIterator;
import javax.jcr.query.qom.Constraint;
import javax.jcr.query.qom.DynamicOperand;
import javax.jcr.query.qom.JoinCondition;
import javax.jcr.query.qom.QueryObjectModel;
import javax.jcr.query.qom.QueryObjectModelConstants;
import javax.jcr.query.qom.QueryObjectModelFactory;
import javax.jcr.query.qom.Source;
import javax.jcr.query.qom.StaticOperand;

import org.apache.commons.lang.StringUtils;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.resource.collection.ResourceCollection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.internal.pdftoolkit.core.util.Utility;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.DamConstants;
import com.day.cq.dam.api.Rendition;
import com.day.cq.dam.api.thumbnail.ThumbnailConfig;
import static com.day.cq.commons.jcr.JcrConstants.NT_FILE;
import static com.day.cq.dam.api.DamConstants.*;

/**
 * This class provides various utility methods pertaining to DAM.
 *
 * @author djaeggi
 * @since CQ 5.4.0
 */
public class DamUtil {
    
    private static final Logger log = LoggerFactory.getLogger(DamUtil.class);

    private static final String[] IMG_MIME_TYPES = {
            DamConstants.THUMBNAIL_MIMETYPE, "image/jpeg", "image/tiff", "image/png",
            "image/bmp", "image/gif", "image/pipeg", "image/x-portable-anymap",
            "image/x-portable-bitmap", "image/x-portable-graymap",
            "image/x-portable-pixmap", "image/x-rgb", "image/x-xbitmap",
            "image/x-xpixmap", "image/x-icon", "image/photoshop",
            "image/x-photoshop", "image/psd", "application/photoshop", "application/psd",
            "image/vnd.adobe.photoshop"};

    private static final String[] VID_MIME_TYPES = { "video/m4v", "video/flv",
        "video/avi", "video/mov", "video/3gpp", "application/x-troff-msvideo",
        "video/vnd", "model/vnd.mts", "video/ts", "video/vnd", "video/dvd",
        "video/x-ms-wmv", "video/msvideo", "video/x-msvideo", "video/x-flv",
        "video/mpeg", "video/x-mpeg", "video/x-m4v", "video/mpg",
        "video/x-mpg", "video/mpeg2", "video/x-mpeg2a", "video/mts",
        "video/x-ms-asf", "video/3gpp2", "video/x-f4v", "video/f4v",
        "video/m2p", "video/mp2t", "video/avchd-stream", "video/m2ts",
        "video/mp2t", "video/vnd.dlna.mpeg-tts", "video/m2v",
        "video/quicktime", "video/x-quicktime", "video/mp4",
        "video/ogg", "video/x-mxf", "application/mxf", "video/x-matroska", "video/mj2",
        "video/vnd.rn-realvideo", "application/vnd.rn-realmedia", "video/webm" };

    private static final List<String> IMG_MIME_TYPE = Arrays.asList(IMG_MIME_TYPES);

    private static final List<String> VID_MIME_TYPE = Arrays.asList(VID_MIME_TYPES);

    private static final String APPLIED_PROCESSING_PROFILE_PROPERTY = JcrConstants.JCR_CONTENT + "/" + DamConstants.PROCESSING_PROFILE;

    private static final int MAGIC_SIZE = 1024;

    /**
     * Translates the path of an asset to its congruent DAM binary path, by replacing the <i>/content/dam</i> path
     * prefix with <i>/var/dam</i>. If the given path does not start with DAM's asset mountpoint, <code>null</code> is
     * returned.
     * <blockquote><pre>
     *   DamUtil.assetToBinaryPath("/content/dam/myfolder/test.jpg")    = "/var/dam/myfolder/test.jpg"
     *   DamUtil.assetToBinaryPath(null)                                = null
     * </pre></blockquote>
     *
     * @param path The path to translate.
     * @return The binary equivalent path or <code>null</code> if the given path is not a DAM asset path.
     * @see com.day.cq.dam.api.DamConstants#MOUNTPOINT_BINARIES
     * @see com.day.cq.dam.api.DamConstants#MOUNTPOINT_ASSETS
     */
    public static String assetToBinaryPath(final String path) {
		return null;
	}

    /**
     * Translates the path of a DAM binary to its congruent asset path, by replacing the <i>/var/dam</i> path prefix
     * with <i>/content/dam</i>. If the given path does not start with DAM's binary mountpoint, <code>null</code> is
     * returned.
     * <blockquote><pre>
     *   DamUtil.binaryToAssetPath("/var/dam/myfolder/test.jpg")        = "/content/dam/myfolder/test.jpg"
     *   DamUtil.binaryToAssetPath(null)                                = null
     * </pre></blockquote>
     *
     * @param path The path to translate.
     * @return The metadata equivalent path or <code>null</code> if the given path is not a DAM binary path.
     * @see com.day.cq.dam.api.DamConstants#MOUNTPOINT_BINARIES
     * @see com.day.cq.dam.api.DamConstants#MOUNTPOINT_ASSETS
     */
    public static String binaryToAssetPath(final String path) {
		return null;
	}

    /**
     * This method determines whether the given {@link Node} represents a thumbnail of an asset.
     *
     * @param file The node to check.
     * @return <code>true</code> if the node represent's an asset's thumbnail, <code>false</code> otherwise.
     */
    public static boolean isThumbnail(Node file) {
		return false;
	}

    /**
     * Returns the name of a thumbnail in the DAM thumbnail name format, respecting the given dimensions of the
     * thumbnail. E.g. providing a <code>width</code> of 100 and a <code>height</code> of 100 would return the thumbnail
     * name <i>cq5dam.thumbnail.100.100.png</i>. As a thumbnail represents a rendition of an {@link
     * com.day.cq.dam.api.Asset}, the name can be used for easy retrieval of the thumbnail:
     * <p/>
     * <blockquote><pre>
     *   ...
     *   final String thumbnailName = DamUtil.getThumbnailName(100, 100);
     *   final Resource thumbnail = asset.getRendition(thumbnailName);
     *   ...
     * </pre></blockquote>
     *
     * @param width  The width of the thumbnail.
     * @param height The height of the thumbnail.
     * @return The rendition name of the thumbnail.
     */
    public static String getThumbnailName(final int width, final int height) {
		return null;
	}

    /**
     * Returns the expected rendition/thumbnail name based on the given thumbnail configuration.
     *
     * @param config The {@link com.day.cq.dam.api.thumbnail.ThumbnailConfig}
     * @return The {@link com.day.cq.dam.api.Rendition} name.
     */
    public static String getThumbnailName(final ThumbnailConfig config) {
		return null;
	}

    /**
     * Behaves like {@link #getThumbnailName(int, int)}. Additionally and optionally a string array of selectors to be
     * added to the thumbnail name can be specified. E.g. providing <code>width</code> = 100, <code>height</code> = 100
     * and <code>selectors</code> = {"a", "b", "c"} would return the thumbnail name
     * <i>cq5dam.thumbnail.100.100.a.b.c.png</i>. Example:
     * <p/>
     * <blockquote><pre>
     *   ...
     *   final String[] selectors = {"a", "b", "c"};
     *   final String thumbnailName = DamUtil.getThumbnailName(100, 100, selectors);
     *   final Resource thumbnail = asset.getRendition(thumbnailName);
     *   ...
     * </pre></blockquote>
     *
     * @param width     The width of the thumbnail.
     * @param height    The height of the thumbnail.
     * @param selectors An array of selectors to be added. May be null (ignored).
     * @return The rendition name of the thumbnail.
     */
    public static String getThumbnailName(final int width, final int height, String[] selectors) {
		return null;
	}

    /**
     * Checks whether the given {@link Resource} represents a {@link com.day.cq.dam.api.Rendition} of an {@link
     * com.day.cq.dam.api.Asset}. The requirements are that given <code>resource</code> is stored within the
     * <i>renditions<i> folder of an asset and that it's node type is <i>nt:file</i>.
     *
     * @param resource The resource to check.
     * @return <code>true</code> if the resource is a rendition.
     */
    public static boolean isRendition(final Resource resource) {
		return false;
	}

    /**
     * Checks whether the given {@link Resource} represents a {@link com.day.cq.dam.api.Asset}. The requirements are
     * that the given <code>resource</code> is of node type <i>dam:Asset</i>.
     *
     * @param resource The resource to check.
     * @return <code>true</code> if the resource is an asset.
     */
    public static boolean isAsset(final Resource resource) {
		return false;
	}
    
    /**
     * Checks whether the give {@link Resource} represents a frozen node. The requirements are 
     * that the given <code>resource</code> is of node type <i>nt:frozenNode</i>.
     * @param resource The resource to check
     * @return <code>true</code> if the resource is an asset.
     */
    public static boolean isFrozenNode(final Resource resource) {
		return false;
	}

    /**
     * Indicates whether the given <code>resource</code> is an {@link Asset}'s sub-asset.
     *
     * @param resource The {@link Resource} to check.
     * @return <code>true</code> if this asset is a sub-asset.
     */
    public static boolean isSubAsset(final Resource resource) {
		return false;
	}

    /**
     * Checks whether the given <code>resource</code> is an asset, and if not, travels upwards the resource hierarchy
     * until a resource is an asset.
     *
     * @param resource The resource to check.
     * @return The {@link Asset} or <code>null</code> if no asset was found.
     */
    public static Asset resolveToAsset(final Resource resource) {
		return null;
	}

    /**
     * This method updates the "last modified" information of the given {@link Asset}.
     *
     * @param asset The asset to update.
     * @param user  The username of who updated the asset.
     * @param date  The date/time the updated happened.
     */
    public static void setModified(final Asset asset, final String user, final Calendar date) {
	}
    
    /**
     * Find all dam assets which are getting expired between lowerBound and upperBound.
     * 
     * @param session
     * @param lowerBound the lower bound of date where asset expiration is computed. If null it is not considered as
     * query criterion  
     * @param upperBound the upper bound of date where asset expiration is computed. If null it is not considered as 
     * query criterion  
     * @throws RepositoryException
     */
    public static List<String> findExpiringAssets(Session session, Calendar lowerBound, Calendar upperBound)
            throws RepositoryException {
				return null;
			}
    
    /**
     * Returns the single property value from the given node <code>n</code> with
     * <code>name</code>. If there is no single-valued property for the given
     * <code>name</code>, then the <code>defaultValue</code> is returned.
     *
     * @param n            a node.
     * @param name         a property name.
     * @param defaultValue the default value.
     * @return the value for the given property name or the default value.
     * @throws RepositoryException if value cannot be retrieved
     */
    public static String getValue(Node n, String name, String defaultValue)
            throws RepositoryException {
				return null;
			}

    /**
     * To check weather asset falls under Image category.
     * @param asset
     * @return True if asset is of type Image.
     */
    public static boolean isImage(Asset asset){
		return false;
	}

    /**
     * To check weather asset falls under Video category.
     * @param asset
     * @return True if asset is of type video.
     */
    public static boolean isVideo(final Asset asset) {
		return false;
	}

    /**
     * Checks if given resource represent a smart collection
     * @param resource an instance of {@link Resource}
     * @return <code>true</code> if <code>resource</code> represents a smart collection
     *         <code>false</code> if <code>resource</code> does not represents a smart collection
     */
    public static boolean isSmartCollection(final Resource resource) {
		return false;
	}

    /**
     * Get a specific Processing profile which may be applied to an asset
     * @param asset The asset we get from payload of DamUpdateAsset Workflow
     * @param profileType The type of profile to get (metadata/video/image)
     * @param session To retrieve nodes from JCR
     * @return {Node} the Processing Profile, null if not found
     */
    public static Node getApplicableProfile(final Asset asset, final String profileType, final Session session) {
		return null;
	}
     /**
     * To get the processing profile path which may be applied to an asset
     * @param asset The asset we get from payload of DamUpdateAsset Workflow
     * @return {String} the path of applied processing profile path
     */
    public static String getAppliedProcessingProfilePath(final Asset asset) {
		return null;
	}

    /**
     * Returns an iterator to all the assets contained in or represented by the resource.
     * If resource is a folder, then all its folders are navigated recursively and assets are listed
     * If a resource is {@link ResourceCollection}, then all its member resources are consulted and navigated recursively if any of the member
     * represent folder or {@link ResourceCollection}.
     * If the resource represents an asset itself, an iterator of single item i.e. Collections.singletonList(res.adaptTo(Asset.class)).iterator();
     * @param res a sling {@link Resource}
     * @return an {@link Iterator} of {@link Asset}
     */
    public static Iterator<Asset> getAssets(Resource res) {
		return null;
	}

    public static boolean checkforAIFile(Asset asset) {
		return false;
	}
    
    public static Rendition getBestFitRendition(int width,
            List<Rendition> renditions) {
				return null;
			}
    
   private static class WidthBasedRenditionComparator implements Comparator<Rendition> {

        public int compare(Rendition r1, Rendition r2) {
			return 0;
		}
    }

    /**
     * A helper class that iterates recursively all child resources of a parent resource (folder or collection) that are asset.
     */
    private static class FolderAssetIterator implements Iterator<Asset> {

        private Resource parentResource;
        private Iterator<Resource> it = null;
        private List<Resource> parentNodes = new ArrayList<Resource>();
        private Asset next = null;

        private Iterator<Resource> getChildren(Resource res) {
			return null;
		}


        public FolderAssetIterator(Resource resource) {
            this.parentResource = resource;
            it = getChildren(this.parentResource);
            next = getNext();
        }

        private Asset getNext() {
			return null;
		}

        public boolean hasNext() {
			return false;
		}

        public Asset next() {
			return null;
		}

        public void remove() {
		}
    }
}
