/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.day.cq.replication;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Map;

import org.apache.sling.event.EventPropertiesMap;
import org.apache.sling.event.EventUtil;
import org.osgi.service.event.Event;

/**
 * Defines the control information of a replication
 */
public class ReplicationAction implements Serializable {

    private static final long serialVersionUID = -151606943486007293L;

    /**
     * Event topic for replication status events.
     */
    public static final String EVENT_TOPIC = "com/day/cq/replication";

    /**
     * The OSGi event property containing the modification date
     */
    public static final String PROPERTY_MODIFICATION_DATE = "modificationDate";

    /**
     * The OSGi event property containing the user id
     */
    public static final String PROPERTY_USER_ID = "userId";

    /**
     * The OSGi event property containing the path
     */
    public static final String PROPERTY_PATH = "path";

    /**
     * The OSGi event property containing the paths
     * @since 5.5
     */
    public static final String PROPERTY_PATHS = "paths";

    /**
     * The OSGi event property containing replication action
     */
    public static final String PROPERTY_TYPE = "type";

    /**
     * The OSGi event property containing the replicated revision (optional)
     */
    public static final String PROPERTY_REVISION = "revision";

    /**
     * Replication path property name.
     */
    public static final String PN_PATH = "cq:repPath";

    /**
     * Replication action type property name.
     */
    public static final String PN_ACTION_TYPE = "cq:repActionType";

    /**
     * the action type
     */
    private final ReplicationActionType type;

    /**
     * all aggregate paths of a replication
     */
    private final String paths[];

    /**
     * The replication time
     */
    private final long time;

    /**
     * The user id triggering the action
     */
    private final String userId;

    /**
     * The revision to replicate (default null)
     */
    private String revision;

    /**
     * Agent config.
     */
    private transient AgentConfig config;

    /**
     * Replication log.
     */
    private transient ReplicationLog log;


    public ReplicationAction(ReplicationActionType type, String path, long time,
                             String userId, String revision) {

        if (type == null) {
            throw new IllegalArgumentException("Type must not be null.");
        }
        if (path == null) {
            throw new IllegalArgumentException("Path must not be null.");
        }
        if (userId == null) {
            throw new IllegalArgumentException("Userid must not be null.");
        }
        this.type = type;
        this.paths = new String[]{path};
        this.time = time == 0 ? System.currentTimeMillis() : time;
        this.userId = userId;
        this.revision = revision;
    }

    public ReplicationAction(ReplicationActionType type, String[] paths, long time,
                             String userId, String revision) {

        if (type == null) {
            throw new IllegalArgumentException("Type must not be null.");
        }
        if (paths == null || paths.length == 0) {
            throw new IllegalArgumentException("Paths must not be null or empty.");
        }
        if (userId == null) {
            throw new IllegalArgumentException("Userid must not be null.");
        }
        this.type = type;
        this.paths = paths;
        this.time = time == 0 ? System.currentTimeMillis() : time;
        this.userId = userId;
        this.revision = revision;
    }

    /**
     * Create a new instance of this class. Used for actions that do not carry
     * content (e.g. DELETE).
     *
     * @param type
     *            action type
     * @param path
     *            path
     */
    public ReplicationAction(ReplicationActionType type, String path) {
        this(type, path, 0, "", null);
    }

    /**
     * Returns the action type
     *
     * @return replication action type
     */
    public ReplicationActionType getType() {
		return null;
	}

    /**
     * Returns the path
     *
     * @return path
     */
    public String getPath() {
		return null;
	}

    /**
     * Returns the replication paths. Currently the array may only contain more than 1 elements for a delete
     * replication. in that case, the paths are the ones of the deleted aggregates (content nodes).
     *
     * @return the paths
     * @since 5.5
     */
    public String[] getPaths() {
		return null;
	}

    /**
     * Returns the revision
     *
     * @return the revision
     */
    public String getRevision() {
		return null;
	}

    /**
     * Returns the replication time
     *
     * @return the replication time
     */
    public long getTime() {
		return 0L;
	}

    /**
     * Returns the user id.
     *
     * @return the user id.
     */
    public String getUserId() {
		return null;
	}

    /**
     * Return the agent config.
     *
     * @return agent config
     */
    public AgentConfig getConfig() {
		return null;
	}

    /**
     * Set the agent config.
     *
     * @param config agent config
     */
    public void setConfig(AgentConfig config) {
	}

    /**
     * Return the replication log.
     *
     * @return replication log
     */
    public ReplicationLog getLog() {
		return null;
	}

    /**
     * Set the replication log.
     *
     * @param log the log
     */
    public void setLog(ReplicationLog log) {
	}

    @Override
    public String toString() {
		return null;
	}

    /**
     * Convert an OSGi event to a replication event.
     *
     * @param evt The OSGi event
     * @return The replication event if the OSGi event contained a replication
     *         event. Otherwise null is returned.
     */
    public static ReplicationAction fromEvent(Event evt) {
		return null;
	}

    /**
     * Create the properties.
     *
     * @return OSGi event properties
     */
    public EventPropertiesMap createProperties() {
		return null;
	}

    /**
     * Create an OSGi event out of the replication event.
     *
     * @return A new OSGi event.
     */
    public Event toEvent() {
		return null;
	}

    /**
     * Create an OSGi event out of the replication event.
     *
     * @param distribute if <code>true</code> a distributed event is created
     * @return A new OSGi event.
     */
    public Event toEvent(boolean distribute) {
		return null;
	}

}
