/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import com.day.cq.search.eval.PredicateEvaluator;
import com.day.cq.search.result.SearchResult;

/**
 * A <code>Predicate</code> is a single constraint for a {@link Query}. It is a
 * pure data model object, consisting of a parameter map based on key-value
 * string pairs.
 * 
 * <p>
 * Each {@link Predicate} has a type ({@link #getType()}) that is used to find
 * the right {@link PredicateEvaluator} that will handle this predicate and eg.
 * translate it into an XPath statement.
 * 
 * <p>
 * Using the composite pattern, the subclass {@link PredicateGroup} allows to
 * group multiple predicates into one. This allows the query to actually consist
 * of a predicate tree and reflects more complex queries that include sub-terms.
 * The methods {@link #getName()}, {@link #getPath()} and {@link #getParent()}
 * return that hierarchy information. The name/path is important, because facets
 * will be returned in a map with the appropriate predicate path as key (see
 * {@link SearchResult#getFacets()}).
 * 
 * <p>
 * The parameters are set via the {@link #set(String, String)} method and
 * retrieved via {@link #get(String)}, {@link #get(String, String)} or
 * {@link #getParameters()}. Predicates are also {@link Cloneable}, so that it
 * is easy to copy them.
 * 
 * @since 5.2
 */
public class Predicate implements Cloneable {
    
    public static final String PARAM_OFFSET = "offset";
    
    public static final String PARAM_LIMIT = "limit";
    
    public static final String PARAM_EXCERPT = "excerpt";

    /**
     * If true, this allows the implementation to optimize the query and return
     * just a guessed {@link SearchResult#getTotalMatches() count}. Defaults to
     * false, in which case the total is always accurate.
     * 
     * @since 5.4
     */
    public static final String PARAM_GUESS_TOTAL = "guessTotal";
    
    public final static String ORDER_BY = "orderby";

    public final static String PARAM_SORT = "sort";

    public final static String SORT_ASCENDING = "asc";

    public final static String SORT_DESCENDING = "desc";

    final private String type;
    
    private String name;
    
    private PredicateGroup parent;
    
    private Map<String, String> params = new HashMap<String, String>();

    private boolean ignore = false;

    /**
     * Creates a {@link Predicate} of the given type. Note that the type
     * cannot be changed later. The name will be deducted automatically,
     * see {@link #getName()}. Initially no parameters will be set and
     * no parent will be present.
     * 
     * @param type
     *            predicate type name for finding the right
     *            {@link PredicateEvaluator}
     */
    public Predicate(String type) {
        this(null, type);
    }

    /**
     * Creates a {@link Predicate} with the given name and type. Note that name
     * and type cannot be changed later. Initially no parameters will be set and
     * no parent will be present.
     * 
     * <p>
     * The name should not be <code>null</code> - this case is reserved for a
     * root {@link PredicateGroup} (using
     * {@link PredicateGroup#PredicateGroup()}).
     * 
     * @param name
     *            identifying name for this predicate
     * @param type
     *            predicate type name for finding the right
     *            {@link PredicateEvaluator}
     */
    public Predicate(String name, String type) {
        this.name = name;
        this.type = type;
    }

    /**
     * Returns the predicate type name for finding the right
     * {@link PredicateEvaluator}.
     */
    public String getType() {
		return null;
	}

    /**
     * Returns the name of this predicate. The name is used to allow round-trip
     * serialization of predicate trees (where the order is encoded in the
     * names) and for identifying predicates in the
     * {@link SearchResult#getFacets() facet map}, in logging or other debug
     * output.
     * 
     * <p>
     * When a predicate is included in {@link PredicateGroup}, the names of the
     * parent predicates will make up the {@link #getPath() path}, eg.
     * <code>parent.child.grandchild</code>. The name must not be changed after
     * the predicate is passed to a {@link Query}.
     * 
     * <p>
     * If no name was set previously, it will be automatically created. If this
     * predicate is part of a group, it will get a name of the form "N_type"
     * where N is the 1-based index of its position in the parent group and
     * "type" is it's {@link #getType() type}. If it does not have a parent group,
     * the name will only consist of the type.
     * 
     * @return identifying name for this predicate (can be <code>null</code> if
     *         no name was set)
     */
    public String getName() {
		return null;
	}

    /**
     * Returns the path in a predicate tree of {@link PredicateGroup
     * PredicateGroups} to this predicate, eg.
     * <code>parent.child.grandchild</code>. This can be <code>null</code> if no
     * name was set. The path is used for identifying predicates in the
     * {@link SearchResult#getFacets() facet map}, in logging or other debug
     * output.
     */
    public String getPath() {
		return null;
	}

    /**
     * Returns the value for the given parameter name or <code>null</code> if
     * that parameter is not set. Parameters always have {@link String} values.
     */
    public String get(String parameterName) {
		return null;
	}

    /**
     * Returns the value for the given parameter name or the given default value
     * if that parameter is not set. Parameters always have {@link String}
     * values.
     */
    public String get(String parameterName, String defaultValue) {
		return null;
	}

    /**
     * Returns the boolean value for the given parameter name or
     * <code>false</code> if that parameter is not set, ie. it assumes
     * <code>false</code> as the default of the boolean parameter.
     * 
     * <p>
     * Since actual parameter values are strings, it is considered
     * <code>true</code> if the string value is either "true" or "on" (used by
     * HTTP forms for checkboxes).
     */
    public boolean getBool(String parameterName) {
		return false;
	}

    /**
     * Sets the parameter 'parameterName' to the value given by 'value'.
     * Parameters have always {@link String} values.
     * 
     * @return returns itself for simple one-liners:
     *         <code>group.add(new Predicate("mytype", "type").set("type", "nt:file"));</code>
     */
    public Predicate set(String parameterName, String value) {
		return null;
	}

    /**
     * Returns <code>true</code> when there is a parameter present with the
     * given name and if the string value is not empty. Parameters always have
     * {@link String} values.
     */
    public boolean hasNonEmptyValue(String parameterName) {
		return false;
	}

    /**
     * Returns an unmodifiable map of all parameters and their values.
     */
    public Map<String, String> getParameters() {
		return null;
	}

    /**
     * This is used only during evaluation and marks this predicate as
     * "invisible" for any {@link PredicateEvaluator} that will encounter it.
     * Don't use that method when creating a query.
     * 
     * @param ignore
     *            <code>true</code> if this predicate should be ignored by
     *            evaluators (default is <code>false</code>)
     */
    public void setIgnored(boolean ignore) {
	}

    /**
     * This is used only during evaluation and indicates that this predicate is
     * marked as "invisible" for any {@link PredicateEvaluator} that will
     * encounter it. This method has no use for clients of the querybuilder API.
     * 
     * @return <code>true</code> if this predicate should be ignored by
     *         evaluators (default is <code>false</code>)
     * 
     */
    public boolean ignored() {
		return false;
	}

    /**
     * Clones this predicate so that the returned clone can be used completely
     * independently from this original. Same as {@link #clone(boolean)
     * clone(false)}.
     */
    @Override
    public Predicate clone() {
		return null;
	}

    /**
     * Clones this predicate so that the returned clone can be used completely
     * independently from this original. Allows for automatic reset of the name.
     * 
     * @param resetName
     *            whether to reset the name to <code>null</code> so that they
     *            will be automatically deducted (see {@link #getName()})
     */
    public Predicate clone(boolean resetName) {
		return null;
	}

    @Override
    public boolean equals(Object obj) {
		return false;
	}

    @Override
    public int hashCode() {
		return 0;
	}
    // -------------------------------------------< protected >

    /**
     * Used by {@link PredicateGroup} to store the hierarchy information when
     * adding a predicate (through eg. {@link PredicateGroup#add(Predicate)}).
     * Normally there is no reason to call this method, hence it is
     * <code>protected</code>.
     * 
     * @param parent
     *            parent predicate group to set
     */
    protected void setParent(PredicateGroup parent) {
	}

    // -------------------------------------------< misc >

    /**
     * Overwrites the standard {@link Object#toString()} implementation and
     * returns a debug-friendly string representation of the predicate.
     */
    public String toString() {
		return null;
	}

}
