/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * A <code>PredicateGroup</code> is a {@link Predicate} that represents a list
 * of {@link Predicate Predicates} (following the composite pattern). This
 * allows to build predicate trees for reflecting complex queries that include
 * sub-terms.
 * 
 * <p>
 * The predicates in a group are by default all required to match, ie. they will
 * be combined with "AND" in a lower-level query language, which is indicated by
 * {@link #allRequired()} returning true. If {@link #setAllRequired(boolean)} is
 * called with <code>false</code>, or if the parameter <code>or</code> is set to
 * <code>true</code>, the child predicates will be combined with "OR", ie. only
 * one must match for the whole group to match.
 * 
 * <p>
 * If the parameter <code>not</code> is set to <code>true</code>, the result of
 * this group will be negated. See also {@link #isNegated()} and
 * {@link #setNegated(boolean)}.
 * 
 * <p>
 * This class extends both the {@link Predicate} class and implements the
 * {@link List} interface, backed by a standard {@link ArrayList} internally.
 * 
 * <p>
 * The standard type name for predicate groups is given by {@link #TYPE} (
 * {@value PredicateGroup#TYPE}), which is also used when using the default
 * constructor.
 * 
 * @since 5.2
 */
public class PredicateGroup extends Predicate implements List<Predicate> {

    public static final String TYPE = "group";
    
    private List<Predicate> predicates = new ArrayList<Predicate>();

    /**
     * Creates this predicate group with the group type
     * <code>{@value #TYPE}</code>, and a predicate name of <code>null</code>.
     * Use this constructor when the name should be deducted automatically (see
     * {@link #getName()}) or for the root group of a predicate tree, because
     * the name must be <code>null</code> for that case.
     */
    public PredicateGroup() {
        super(null, PredicateGroup.TYPE);
    }

    /**
     * Creates this predicate group with the given name and the group type
     * <code>{@value #TYPE}</code>, using the
     * {@link Predicate#Predicate(String, String)} constructor. If you create a
     * root group of a predicate tree, the name must be <code>null</code> (you
     * can use the default constructor {@link #PredicateGroup()} for that).
     */
    public PredicateGroup(String name) {
        super(name, PredicateGroup.TYPE);
    }
    
    /**
     * Converts a map with predicates and their parameters into a predicate
     * tree. Accepts a map with strings as keys and either simple strings as
     * values or string arrays as values. In the array case, the first value
     * will be chosen.
     * 
     * <p>
     * Same as {@link PredicateConverter#createPredicates(Map)}.
     */
    public static PredicateGroup create(Map predicateParameterMap) {
		return null;
	}
    
    /**
     * Returns an URL query part containing the given group. This is the same
     * mapping as used in {@link #createMap(PredicateGroup)} and
     * {@link #createPredicates(Map)}. For example, the returned value could be:
     * <code>type=cq:Page&path=/content</code>. Note that this won't be a
     * complete URL, just a list of parameters for an URL query part. The keys
     * and values will be properly escaped for use in an URL.
     * 
     * <p>
     * Same as {@link PredicateConverter#toURL(PredicateGroup)}.
     */
    public String toURL() {
		return null;
	}

    /**
     * Returns whether all predicates are combined with "AND", ie. only nodes
     * are found that match all predicates in this group. The default value is
     * <code>true</code> (AND):
     * 
     * @return <code>true</code> for "AND" (default), <code>false</code> for "OR"
     */
    public boolean allRequired() {
		return false;
	}

    /**
     * Sets whether all predicates are combined with "AND", ie. only nodes are
     * found that match all predicates in this group, or if they are combined
     * with "OR".
     * 
     * @param all
     *            <code>true</code> for "AND", <code>false</code> for "OR"
     */
    public void setAllRequired(boolean all) {
	}

    /**
     * Returns whether the result of this predicate group should be negated. Ie.
     * only nodes that do not match this group should be included in the
     * results. The default value is <code>false</code>.
     * 
     * @return <code>true</code> for exclusive, <code>false</code> for inclusive
     *         (default)
     * 
     * @since 5.5
     */
    public boolean isNegated() {
		return false;
	}

    /**
     * Sets whether the result of this group should be negated.
     * 
     * @param not
     *            <code>true</code> if the group should be negated,
     *            <code>false</code> if not (default)
     * 
     * @since 5.5
     */
    public void setNegated(boolean not) {
	}

    /**
     * Returns a certain predicate by its {@link Predicate#getName() name}.
     */
    public Predicate getByName(String name) {
		return null;
	}

    /**
     * Returns a certain predicate by its {@link Predicate#getPath() path},
     * relative to this predicate.
     */
    public Predicate getByPath(String path) {
		return null;
	}
    
    /**
     * Clones this predicate group so that the returned clone can be used
     * completely independently from this original. All child predicates
     * will hence also be cloned. 
     */
    @Override
    public PredicateGroup clone() {
		return null;
	}

    /**
     * Clones this predicate group so that the returned clone can be used
     * completely independently from this original. All child predicates will
     * hence also be cloned. A new name for the clone can be passed.
     * 
     * @param resetName
     *            whether to reset the name and child predicate names to
     *            <code>null</code> so that they will be automatically
     *            deducted (see {@link #getName()})
     */
    @Override
    public PredicateGroup clone(boolean resetName) {
		return null;
	}

    @Override
    public boolean equals(Object obj) {
		return false;
	}

    @Override
    public int hashCode() {
		return 0;
	}

    // -------------------------------------------< misc >
    
    private static int indent = 0;

    /**
     * Overwrites the standard {@link Object#toString()} implementation and
     * returns a debug-friendly string including all sub predicates (via their
     * {@link Predicate#toString() toString()} method). The final string is
     * multi-lined and indented for easy readability of the inherent tree
     * structure.
     */
    public String toString() {
		return null;
	}
    
    private void appendIndent(StringBuffer buffer) {
	}
    
    protected void setMeAsParent(Predicate element) {
	}

    protected void unsetParent(Predicate element) {
	}

    // ------------------------------------------------< List interface >

    public boolean add(Predicate o) {
		return false;
	}

    public void add(int index, Predicate element) {
	}

    public boolean addAll(Collection<? extends Predicate> c) {
		return false;
	}

    public boolean addAll(int index, Collection<? extends Predicate> c) {
		return false;
	}

    public void clear() {
	}

    public boolean contains(Object o) {
		return false;
	}

    public boolean containsAll(Collection<?> c) {
		return false;
	}

    public Predicate get(int index) {
		return null;
	}

    public int indexOf(Object o) {
		return 0;
	}

    public boolean isEmpty() {
		return false;
	}

    public Iterator<Predicate> iterator() {
		return null;
	}

    public int lastIndexOf(Object o) {
		return 0;
	}

    public ListIterator<Predicate> listIterator() {
		return null;
	}

    public ListIterator<Predicate> listIterator(int index) {
		return null;
	}

    public boolean remove(Object o) {
		return false;
	}

    public Predicate remove(int index) {
		return null;
	}

    public boolean removeAll(Collection<?> c) {
		return false;
	}

    public boolean retainAll(Collection<?> c) {
		return false;
	}

    public Predicate set(int index, Predicate element) {
		return null;
	}

    public int size() {
		return 0;
	}

    public List<Predicate> subList(int fromIndex, int toIndex) {
		return null;
	}

    public Object[] toArray() {
		return null;
	}

    public <T> T[] toArray(T[] a) {
		return null;
	}
}
