/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.security.util;

import com.day.cq.replication.Replicator;
import org.apache.jackrabbit.JcrConstants;
import org.apache.jackrabbit.api.security.JackrabbitAccessControlEntry;
import org.apache.jackrabbit.api.security.JackrabbitAccessControlList;
import org.apache.jackrabbit.api.security.JackrabbitAccessControlManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.Principal;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Collection;
import java.util.Arrays;

import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.security.AccessControlEntry;
import javax.jcr.security.AccessControlException;
import javax.jcr.security.AccessControlManager;
import javax.jcr.security.AccessControlPolicy;
import javax.jcr.security.AccessControlPolicyIterator;
import javax.jcr.security.Privilege;

/**
 * This class defines the main CQ Actions and provides the mapping from and to
 * JCR Privileges.
 *
 * <pre>
 * "read"           Privilege.JCR_READ
 *
 * "modify"         Privilege.JCR_MODIFY_PROPERTIES,
 *                  Privilege.JCR_LOCK_MANAGEMENT,
 *                  Privilege.JCR_VERSION_MANAGEMENT
 *
 * "create"         Privilege.JCR_ADD_CHILD_NODES,
 *                  Privilege.JCR_NODE_TYPE_MANAGEMENT
 *
 * "delete"         Privilege.JCR_REMOVE_CHILD_NODES,
 *                  Privilege.JCR_REMOVE_NODE
 *
 * "acl_read"       Privilege.JCR_READ_ACCESS_CONTROL
 *
 * "acl_write"      Privilege.JCR_MODIFY_ACCESS_CONTROL
 *
 * "replicate"      "crx:replicate" (custom privilege)
 *
 * </pre>
 */
public class CqActions {

    private static Logger log = LoggerFactory.getLogger(CqActions.class);

    /**
     * CQ actions constants
     */
    public static final String[] ACTIONS = new String[] {
            "read", "modify", "create", "delete", "acl_read", "acl_edit", "replicate"
    };

    private static final String CONTENT_RESTRICTION = "*/jcr:content*";


    private final Session session;
    private final Map<String, Set<Privilege>> map = new HashMap<String, Set<Privilege>>();

    public CqActions(Session session) throws RepositoryException {
        this.session = session;
        AccessControlManager acMgr = session.getAccessControlManager();
        map.put("read", getPrivilegeSet(Privilege.JCR_READ, acMgr));
        map.put("modify", getPrivilegeSet(new String[] {
                /* privilege to add/modify/remove properties, lock the target
                   node and execute version operations on it (ci, co, restore...) */
                Privilege.JCR_MODIFY_PROPERTIES,
                Privilege.JCR_LOCK_MANAGEMENT,
                Privilege.JCR_VERSION_MANAGEMENT}, acMgr));
        map.put("create", getPrivilegeSet(new String[] {
                /* can add any kind of nodes below the target node.
                   nt_mgt is required for calls like Node.addNode(name, ntName)
                   or adding mixins */
                Privilege.JCR_ADD_CHILD_NODES,
                Privilege.JCR_NODE_TYPE_MANAGEMENT}, acMgr));
        map.put("delete", getPrivilegeSet(new String[] {
                /* privileges required to remove any kind of nodes below the
                   target node.
                   NOTE: this does NOT include the permission to remove the target
                   node itself as JCR_REMOVE_CHILD_NODES would be required on
                   the parent node.
                   This is - AFAIK - different compared to the behaviour of cq 5.2
                   After discussion with David, we decided that removing nodes
                   below target is sufficient and most probably what is the
                   desired effect. Otherwise granting delete would require
                   privilege modifications on 2 different nodes that may also
                   have different edit_acl permissions... :(
                 */
                Privilege.JCR_REMOVE_CHILD_NODES,
                Privilege.JCR_REMOVE_NODE}, acMgr));
        map.put("acl_read", getPrivilegeSet(Privilege.JCR_READ_ACCESS_CONTROL, acMgr));
        map.put("acl_edit", getPrivilegeSet(Privilege.JCR_MODIFY_ACCESS_CONTROL, acMgr));

        try {
            map.put("replicate", getPrivilegeSet(Replicator.REPLICATE_PRIVILEGE, acMgr));
        } catch (AccessControlException e) {
            log.warn("Replicate privilege not registered");
        }

        // NOTE: JCR_LIFECYCLE_MANAGEMENT and JCR_RETENTION_MANAGEMENT have been
        //       intentionally omitted. the first isn't used within the scope of
        //       CQ and the latter is expected to be present with defined
        //       retention management tools only.
    }

    /**
     * Returns the privileges that correspond to the given <code>action</code> string.
     *
     * @param action The action to be mapped.
     * @return A set of privileges.
     * @deprecated As of CQ 5.4 the mapping of CQ action to resulting ACEs/privileges
     * depends on the nature of the target node.
     */
    public Set<Privilege> getPrivileges(String action) {
		return null;
	}

    /**
     * Tests if the given <code>action</code> is granted by the given set of
     * privileges.
     *
     * @param action the action to tested
     * @param privs the privileges (on the node)
     * @return <code>true</code> if the action is allowed.
     * @deprecated As of CQ 5.4 the mapping of CQ action to privileges depends on
     * the nature of the target node.
     */
    public boolean isGranted(Set<Privilege> privs, String action) {
		return false;
	}

    /**
     * Returns the names of the actions that are granted on the given path.
     *
     * @param path the path to check
     * @param session the session
     * @return the set of actions that are granted.
     * @throws RepositoryException if an error occurs
     * @deprecated Since 5.4 
     */
    public Collection<String> getActions(Session session, String path) throws RepositoryException {
		return null;
	}

    /**
     *
     * @param nodePath
     * @param principals The set of principals for which the action set needs
     * to be evaluated.
     * @return the set of actions granted for the specified principals at the
     * specified path.
     * @throws RepositoryException If an error occurs.
     * @since 5.4
     */
    public Collection<String> getAllowedActions(String nodePath, Set<Principal> principals) throws RepositoryException {
		return null;
	}

    /**
     * Installs the specified actions for the given principal at the specified
     * targetNode by converting it the corresponding JCR access control content.
     *
     * @param nodePath
     * @param actionMap A map of CQ Action name to a Boolean indicating whether
     * the action should be granted or denied.
     * @param inheritedAllows
     * @throws RepositoryException If an error occurs.
     */
    public void installActions(String nodePath, Principal principal, Map<String,
            Boolean> actionMap, Collection<String> inheritedAllows) throws RepositoryException {
			}

    /**
     * Returns <code>true</code> if the node is defined to have a jcr:content
     * child node (that may or may not be present yet). Note that the test
     * by intention does not rely on the existence of a jcr:content node
     * that may as well be present with an unstructured or folder node.
     *
     * @param node
     * @return true if the specified node is defined to possibly have a
     * jcr:content child (such as e.g. nt:file, cq:Page and similar).
     * @throws RepositoryException
     */
    public static boolean definesContent(Node node) throws RepositoryException {
		return false;
	}

    /**
     * 
     * @param ace
     * @return
     * @throws RepositoryException
     */
    public static boolean hasContentRestriction(AccessControlEntry ace) throws RepositoryException {
		return false;
	}
    
    //------------------------------------------------------------< private >---

    private static Set<Privilege> getPrivileges(String path, Set<Principal> principals, AccessControlManager acMgr) throws RepositoryException, AccessDeniedException {
		return null;
	}

    private static Set<Privilege> getPrivilegeSet(String privName, AccessControlManager acMgr) throws RepositoryException {
		return null;
	}

    private static Set<Privilege> getPrivilegeSet(String[] privNames, AccessControlManager acMgr) throws RepositoryException {
		return null;
	}

    private static JackrabbitAccessControlList getModifiableAcl(AccessControlManager acMgr, String path)
            throws RepositoryException, AccessDeniedException {
				return null;
			}
}