/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.workflow.process;

import com.day.cq.replication.ReplicationActionType;
import com.day.cq.replication.ReplicationException;
import com.day.cq.replication.ReplicationOptions;
import com.day.cq.replication.Replicator;
import com.day.cq.wcm.workflow.api.WcmWorkflowService;
import com.day.cq.workflow.WorkflowException;
import com.day.cq.workflow.WorkflowService;
import com.day.cq.workflow.WorkflowSession;
import com.day.cq.workflow.collection.ResourceCollection;
import com.day.cq.workflow.collection.ResourceCollectionManager;
import com.day.cq.workflow.collection.ResourceCollectionUtil;
import com.day.cq.workflow.exec.HistoryItem;
import com.day.cq.workflow.exec.WorkItem;
import com.day.cq.workflow.exec.WorkflowData;
import com.day.cq.workflow.exec.WorkflowProcess;
import com.day.cq.workflow.metadata.MetaDataMap;
import com.day.cq.workflow.model.WorkflowNode;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.sling.jcr.api.SlingRepository;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventAdmin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.jcr.security.AccessControlManager;
import javax.jcr.security.Privilege;

import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.List;

/**
 * This abstract <code>ReplicatePageProcess</code> class serves as the basis for
 * all replication specific <code>JavaProcess</code> classes, like activate,
 * deactivate...<br>
 * This Porcess starts the Replicator with the type provided by the implementing
 * Classes
 * {@link com.day.cq.wcm.workflow.process.ReplicatePageProcess#getReplicationType()
 * getReblicationType()-method} of payloads of type <i>path</i> or <i>uuid</i><br>
 * This Process checks permissions.<br>
 * In case the {@link com.day.cq.workflow.WorkflowSession Session} starting the
 * process is lacking the {@link com.day.cq.security.privileges.Privilege
 * Privilege} to replicate, an event of this topic
 * <code>{@value com.day.cq.wcm.workflow.api.WcmWorkflowService#EVENT_TOPIC}</code>
 * is send out. Listeners to this topic, may handle this situation.
 * <p/>
 * <p/>
 * <b>Configuration</b>
 * This process supports the following configuration arguments:
 * <dl>
 * <dt>replicateAsParticipant</dt>
 * <dd>Boolean flag indicating if the replication should be performed under the context of the latest participant, Default is <code>false</code>,
 * meaning the replication is performed as admin. This process supports this configuration either as a dedicated process
 * arguments or as part of the generic PROCESS_ARGS argument. If set to <code>true</code> the workflow model must have
 * a participant or daynamic participant step modeled ahead this replication process, to determine the participant. If no
 * participant can be determined, it falls back to admin.
 * </dd>
 * </dl>
 *
 */
@Component(componentAbstract = true)
public abstract class ReplicatePageProcess implements WorkflowProcess {
    /**
     * the logger
     */
    private static final Logger log = LoggerFactory.getLogger(ReplicatePageProcess.class);

    public static final String TYPE_JCR_PATH = "JCR_PATH";
    public static final String TYPE_JCR_UUID = "JCR_UUID";

    @Reference
    protected Replicator replicator;

    @Reference
    protected WcmWorkflowService wcmWfService;

    @Reference
    protected WorkflowService wfService;

    @Reference
    protected EventAdmin eventAdmin;

    @Reference
    protected ResourceCollectionManager rcManager;

    @Reference
    private SlingRepository repository;

    /**
     * admin jcr session
     */
    private Session admin;

    private enum Arguments {
        PROCESS_ARGS("PROCESS_ARGS"), REPLICATE_AS_PARTICIPANT("replicateAsParticipant");

        private String argumentName;

        Arguments(String argumentName) {
            this.argumentName = argumentName;
        }

        public String getArgumentName() {
            return this.argumentName;
        }
    }

    /**
     * @see com.day.cq.workflow.exec.WorkflowProcess#execute(com.day.cq.workflow.exec.WorkItem,
     *      com.day.cq.workflow.WorkflowSession,
     *      com.day.cq.workflow.metadata.MetaDataMap)
     */
    public void execute(WorkItem workItem, WorkflowSession workflowSession, MetaDataMap args) throws WorkflowException {
	}

    /**
     * Determine the replication mode from the arguments map.
     *
     * @param args The process arguments
     * @return Depending on the process arguments <code>true</code> if the replication is supposed to take place under
     *         the participants context, <code>false</code> otherwise.
     */
    private boolean replicateAsParticipant(MetaDataMap args) {
		return false;
	}

    /**
     * Get a session for the given approver.
     *
     * @param participantId
     * @param workflowSession
     * @return The approver's session or <code>null</code> in case of repository exceptions.
     */

    private Session getParticipantSession(String participantId, WorkflowSession workflowSession) {
		return null;
	}

    /**
     * Travael up the session's history to find the latest participant step or dynamic participant step and use it's
     * current assignee as approver.
     *
     * @param workItem
     * @param workflowSession
     * @return The approver's id of the latest participant/dynamic participant step in the history. In case there is no
     *         participant step, <code>null</code> is returned.
     */
    private String resolveParticipantId(WorkItem workItem, WorkflowSession workflowSession) {
		return null;
	}

    private List<String> getPaths(String path, ResourceCollection rcCollection) {
		return null;
	}

    /**
     * Specifies the <code>{@link ReplicationActionType}</code> for which this
     * class is designed for.
     *
     * @return <code>{@link ReplicationActionType}</code>
     */
    public abstract ReplicationActionType getReplicationType() {
		return null;
	}

    /**
     * allows subclasses to mangle with the replication options.
     *
     * @return the options or <code>null</code>
     */
    protected ReplicationOptions prepareOptions(ReplicationOptions opts) {
		return null;
	}

    protected boolean canReplicate(Session session, String path) throws AccessDeniedException {
		return false;
	}

    // ---------- SCR Integration ----------------------------------------------

    protected void activate(ComponentContext context) throws RepositoryException {
	}

    protected void deactivate(ComponentContext context) {
	}
}
