/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.commons.visitor;

import java.util.LinkedList;

import javax.jcr.Item;
import javax.jcr.ItemVisitor;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.RepositoryException;

import org.apache.jackrabbit.commons.predicate.Predicate;

public abstract class FilteringItemVisitor implements ItemVisitor {

    /**
     * Predicate that defines which items are included.
     */
    protected Predicate includePredicate = Predicate.TRUE;

    /**
     * Predicate that defines which items are traversed.
     */
    protected Predicate traversalPredicate = Predicate.TRUE;

    /**
     * Do we want to walk all properties of nodes?
     * The default is false.
     */
    protected boolean walkProperties = false;

    /**
     * indicates if traversal should be done in a breadth-first
     * manner rather than depth-first (which is the default)
     */
    protected boolean breadthFirst = false;

    /**
     * the 0-based level up to which the hierarchy should be traversed
     * (if it's -1, the hierarchy will be traversed until there are no
     * more children of the current item)
     */
    protected int maxLevel = -1;

    /**
     * queues used to implement breadth-first traversal
     */
    protected LinkedList currentQueue;
    protected LinkedList nextQueue;

    /**
     * used to track hierarchy level of item currently being processed
     */
    protected int currentLevel;

    public void setMaxLevel(final int ml) {
	}

    public void setBreadthFirst(final boolean flag) {
	}
    public void setWalkProperties(final boolean flag) {
	}

    public void setIncludePredicate(final Predicate ip) {
	}

    public void setTraversalPredicate(final Predicate tp) {
	}

    /**
     * Implement this method to add behaviour performed before a
     * <code>Property</code> is visited.
     *
     * @param property the <code>Property</code> that is accepting this visitor.
     * @param level    hierarchy level of this property (the root node starts at level 0)
     * @throws RepositoryException if an error occurrs
     */
    protected abstract void entering(Property property, int level)
        throws RepositoryException {
	}

    /**
     * Implement this method to add behaviour performed before a
     * <code>Node</code> is visited.
     *
     * @param node  the <code>Node</code> that is accepting this visitor.
     * @param level hierarchy level of this node (the root node starts at level 0)
     * @throws RepositoryException if an error occurrs
     */
    protected abstract void entering(Node node, int level)
        throws RepositoryException {
	}

    /**
     * Implement this method to add behaviour performed after a
     * <code>Property</code> is visited.
     *
     * @param property the <code>Property</code> that is accepting this visitor.
     * @param level    hierarchy level of this property (the root node starts at level 0)
     * @throws RepositoryException if an error occurrs
     */
    protected abstract void leaving(Property property, int level)
        throws RepositoryException {
	}

    /**
     * Implement this method to add behaviour performed after a
     * <code>Node</code> is visited.
     *
     * @param node  the <code>Node</code> that is accepting this visitor.
     * @param level hierarchy level of this node (the root node starts at level 0)
     * @throws RepositoryException if an error occurrs
     */
    protected abstract void leaving(Node node, int level)
        throws RepositoryException {
	}

    /**
     * Called when the Visitor is passed to a <code>Property</code>.
     * <p/>
     * It calls <code>TraversingItemVisitor.entering(Property, int)</code> followed by
     * <code>TraversingItemVisitor.leaving(Property, int)</code>. Implement these abstract methods to
     * specify behaviour on 'arrival at' and 'after leaving' the <code>Property</code>.
     * <p/>
     * <p/>
     * If this method throws, the visiting process is aborted.
     *
     * @param property the <code>Property</code> that is accepting this visitor.
     * @throws RepositoryException if an error occurrs
     */
    public void visit(Property property) throws RepositoryException {
	}

    /**
     * Called when the Visitor is passed to a <code>Node</code>.
     * <p/>
     * It calls <code>TraversingItemVisitor.entering(Node, int)</code> followed by
     * <code>TraversingItemVisitor.leaving(Node, int)</code>. Implement these abstract methods to
     * specify behaviour on 'arrival at' and 'after leaving' the <code>Node</code>.
     * <p/>
     * If this method throws, the visiting process is aborted.
     *
     * @param node the <code>Node</code> that is accepting this visitor.
     * @throws RepositoryException if an error occurrs
     */
    public void visit(Node node)
    throws RepositoryException {
	}
}
