/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.kernel;

import static com.google.common.base.Preconditions.checkNotNull;
import static java.util.Collections.emptyList;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.EMPTY_NODE;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.MISSING_NODE;
import static org.apache.jackrabbit.oak.plugins.memory.PropertyStates.createProperty;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import javax.annotation.Nonnull;
import javax.jcr.PropertyType;

import com.google.common.base.Function;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

import org.apache.jackrabbit.mk.api.MicroKernel;
import org.apache.jackrabbit.mk.api.MicroKernelException;
import org.apache.jackrabbit.oak.commons.json.JsopReader;
import org.apache.jackrabbit.oak.commons.json.JsopTokenizer;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.commons.PathUtils;
import org.apache.jackrabbit.oak.plugins.memory.BinaryPropertyState;
import org.apache.jackrabbit.oak.plugins.memory.BooleanPropertyState;
import org.apache.jackrabbit.oak.plugins.memory.DoublePropertyState;
import org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState;
import org.apache.jackrabbit.oak.plugins.memory.LongPropertyState;
import org.apache.jackrabbit.oak.plugins.memory.MemoryNodeBuilder;
import org.apache.jackrabbit.oak.plugins.memory.PropertyStates;
import org.apache.jackrabbit.oak.plugins.memory.StringPropertyState;
import org.apache.jackrabbit.oak.plugins.value.Conversions;
import org.apache.jackrabbit.oak.spi.state.AbstractChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.AbstractNodeState;
import org.apache.jackrabbit.oak.spi.state.ChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.apache.jackrabbit.oak.spi.state.NodeStateDiff;

/**
 * Basic {@link NodeState} implementation based on the {@link MicroKernel}
 * interface. This class makes an attempt to load data lazily.
 */
public final class KernelNodeState extends AbstractNodeState {

    /**
     * Maximum number of child nodes kept in memory.
     */
    public static final int MAX_CHILD_NAMES = 100;

    /**
     * Number of child nodes beyond which {@link MicroKernel#diff(String, String, String, int)}
     * is used for diffing.
     */
    public static final int LOCAL_DIFF_THRESHOLD = 10;

    /**
     * Dummy cache instance for static {@link #NULL} kernel node state.
     */
    private static final LoadingCache<String, KernelNodeState> DUMMY_CACHE =
        CacheBuilder.newBuilder().build(new CacheLoader<String, KernelNodeState>() {
            @Override
            public KernelNodeState load(String key) throws Exception {
                throw new UnsupportedOperationException();
            }
        });

    /**
     * This {@code NULL} kernel node state is used as a value in the
     * {@link #cache} to indicate that there is no node state at the given
     * path and revision. This object is only used internally and never leaves
     * this {@link KernelNodeState}.
     */
    private static final KernelNodeState NULL = new KernelNodeState();

    private final KernelNodeStore store;

    private final MicroKernel kernel;

    private final String path;

    private String revision;

    private Map<String, PropertyState> properties;

    private long childNodeCount = -1;

    private long childNodeCountMin;

    private String hash;

    private String id;

    private Set<String> childNames;

    /**
     * {@code true} is this is a node state from a branch. {@code false}
     * otherwise.
     * <p>
     * FIXME: this is a workaround to avoid creating branches from a branch
     * until this is supported by the MicroKernel. See {@link KernelNodeState#builder()}.
     */
    private boolean isBranch;

    private final LoadingCache<String, KernelNodeState> cache;

    /**
     * Create a new instance of this class representing the node at the
     * given {@code path} and {@code revision}. It is an error if the
     * underlying Microkernel does not contain such a node.
     *
     * @param store the underlying KernelNodeStore
     * @param path the path of this KernelNodeState
     * @param revision the revision of the node to read from the kernel.
     * @param cache the KernelNodeState cache
     */
    public KernelNodeState(
            KernelNodeStore store, String path, String revision,
            LoadingCache<String, KernelNodeState> cache) {
        this.store = store;
        this.kernel = store.getKernel();
        this.path = checkNotNull(path);
        this.revision = checkNotNull(revision);
        this.cache = checkNotNull(cache);
    }

    private KernelNodeState() {
        this.store = null;
        this.kernel = null;
        this.path = "null";
        this.revision = "null";
        this.cache = DUMMY_CACHE;
    }

    private void init() {
	}

    @Override
    public boolean exists() {
		return false;
	}

    @Override
    public long getPropertyCount() {
		return 0L;
	}

    @Override
    public boolean hasProperty(String name) {
		return false;
	}

    @Override
    public PropertyState getProperty(String name) {
		return null;
	}

    @Override
    public Iterable<? extends PropertyState> getProperties() {
		return null;
	}
    
    @Override
    public long getChildNodeCount(long max) {
		return 0L;
	}

    @Override
    public boolean hasChildNode(String name) {
		return false;
	}

    @Override
    public NodeState getChildNode(String name) {
		return null;
	}

    @Override
    public Iterable<? extends ChildNodeEntry> getChildNodeEntries() {
		return null;
	}

    private Iterable<ChildNodeEntry> getChildNodeEntries(final long offset) {
		return null;
	}

    /**
     * This implementation returns a {@link KernelNodeBuilder} unless this is not
     * a root node or {@link #isBranch} is {@code true} in which case a
     * {@link MemoryNodeBuilder} is returned.
     * <p>
     * TODO: this is a workaround to avoid creating branches from a branch
     * until this is supported by the MicroKernel.
     */
    @Override
    public NodeBuilder builder() {
		return null;
	}

    /**
     * Optimised comparison method that can avoid traversing all properties
     * and child nodes if both this and the given base node state come from
     * the same MicroKernel and either have the same content hash (when
     * available) or are located at the same path in different revisions.
     *
     * @see <a href="https://issues.apache.org/jira/browse/OAK-175">OAK-175</a>
     */
    @Override
    public boolean compareAgainstBaseState(NodeState base, NodeStateDiff diff) {
		return false;
	}

    //------------------------------------------------------------< Object >--

    /**
     * Optimised equality check that can avoid a full tree comparison if
     * both instances come from the same MicroKernel and have either
     * the same revision and path or the same content hash (when available).
     * Otherwise we fall back to the default tree comparison algorithm.
     *
     * @see <a href="https://issues.apache.org/jira/browse/OAK-172">OAK-172</a>
     */
    @Override
    public boolean equals(Object object) {
		return false;
	}

    //------------------------------------------------------------< internal >---

    @Nonnull
    String getRevision() {
		return null;
	}

    /**
     * Mark this instance as from being on branch.
     * <p>
     * TODO this is a workaround to avoid creating branches from a branch
     * until this is supported by the MicroKernel. See {@link KernelNodeState#builder()}.
     * @return {@code this}
     */
    KernelNodeState setBranch() {
		return null;
	}

    /**
     * @return  {@code true} if this instance has been marked as being on a branch by a call
     * to {@link #setBranch()}
     */
    boolean isBranch() {
		return false;
	}

    /**
     * @return the approximate memory usage of this node state.
     */
    synchronized int getMemory() {
		return 0;
	}

    //------------------------------------------------------------< private >---

    private static boolean hasChanges(String journal) {
		return false;
	}

    /**
     * Process the given JSON diff, which is the diff of of the {@code base}
     * node state to this node state.
     *
     * @param jsonDiff the JSON diff.
     * @param base the base node state.
     * @param diff where diffs are reported to.
     * @return {@code true} to continue the comparison, {@code false} to stop
     */
    private boolean processJsonDiff(String jsonDiff,
                                 KernelNodeState base,
                                 NodeStateDiff diff) {
									return false;
								}

    private Iterable<ChildNodeEntry> iterable(
            Iterable<String> names) {
				return null;
			}

    private class KernelChildNodeEntry extends AbstractChildNodeEntry {

        private final String name;

        /**
         * Creates a child node entry with the given name.
         *
         * @param name child node name
         */
        public KernelChildNodeEntry(String name) {
            this.name = checkNotNull(name);
        }

        @Override
        public String getName() {
			return null;
		}

        @Override
        public NodeState getNodeState() {
			return null;
		}

    }

    /**
     * Read a {@code PropertyState} from a {@link JsopReader}
     * @param name  The name of the property state
     * @param reader  The reader
     * @return new property state
     */
    private PropertyState readProperty(String name, JsopReader reader) {
		return null;
	}

    /**
     * Read a multi valued {@code PropertyState} from a {@link JsopReader}
     * @param name  The name of the property state
     * @param reader  The reader
     * @return new property state
     */
    private PropertyState readArrayProperty(String name, JsopReader reader) {
		return null;
	}
    
    @Override
    public String toString() {
		return null;
	}

}
