/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.memory;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.collect.Maps.newHashMap;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.MISSING_NODE;

import java.util.Map;
import java.util.Map.Entry;

import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.spi.state.AbstractNodeState;
import org.apache.jackrabbit.oak.spi.state.ChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.EqualsDiff;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.apache.jackrabbit.oak.spi.state.NodeStateDiff;

/**
 * A <em>mutable</em> state being built.
 *
 * Instances of this class are never passed beyond the containing
 * {@link MemoryNodeBuilder}, so it's not a problem that we intentionally
 * break the immutability assumption of the
 * {@link org.apache.jackrabbit.oak.spi.state.NodeState} interface.
 */
class MutableNodeState extends AbstractNodeState {

    /**
     * The immutable base state.
     */
    private NodeState base;

    /**
     * Set of added, modified or removed ({@code null} value)
     * property states.
     */
    private final Map<String, PropertyState> properties = newHashMap();

    /**
     * Set of added, modified or removed (non-existent value)
     * child nodes.
     */
    private final Map<String, MutableNodeState> nodes = newHashMap();

    MutableNodeState(@Nonnull NodeState base) {
        checkNotNull(base);
        this.base = ModifiedNodeState.unwrap(base, properties, nodes);
    }

    NodeState snapshot() {
		return null;
	}

    void reset(NodeState newBase) {
	}

    /**
     * Equivalent to
     * <pre>
     *   MutableNodeState child = getChildNode(name, true);
     *   child.reset(state);
     *   return child;
     * </pre>
     *
     * @throws IllegalArgumentException if the given name string is empty
     *                                  or contains the forward slash character
     */
    @Nonnull
    MutableNodeState setChildNode(String name, NodeState state)
            throws IllegalArgumentException {
				return null;
			}

    /**
     * Determine whether this node state is modified wrt. the passed
     * {@code before} state.
     * <p>
     * A node state is modified if it either has not the same properties
     * or has not the same child nodes as a {@code before} state. A node
     * state has the same properties as a {@code before} state iff its
     * set of properties is equal to the set of properties of
     * {@code before}. A node state has the same child nodes as a
     * {@code before} state iff its set of child node names is equal to
     * the set of child node names of {@code before}.
     */
    boolean isModified(NodeState before) {
		return false;
	}

    boolean isReplaced(NodeState before) {
		return false;
	}

    boolean isReplaced(NodeState before, String name) {
		return false;
	}

    /**
     * Remove the child node with the given {@code name}.
     * @param name  name of the child node to remove
     * @return  {@code true} if a child node {@code name} existed, {@code false} otherwise.
     */
    boolean removeChildNode(String name) {
		return false;
	}

    /**
     * Remove the property of the given {@code name}.
     * @param name  name of the property to remove
     * @return  {@code true} if a property {@code name} existed, {@code false} otherwise.
     */
    boolean removeProperty(String name) {
		return false;
	}

    /**
     * Set the value of a property
     */
    void setProperty(PropertyState property) {
	}

    @Override
    public String toString() {
		return null;
	}

    //-----------------------------------------------------< NodeState >--

    @Override
    public boolean exists() {
		return false;
	}

    @Override
    public long getPropertyCount() {
		return 0L;
	}

    @Override
    public boolean hasProperty(String name) {
		return false;
	}

    @Override
    public PropertyState getProperty(String name) {
		return null;
	}

    @Override @Nonnull
    public Iterable<? extends PropertyState> getProperties() {
		return null;
	}

    @Override
    public long getChildNodeCount(long max) {
		return 0L;
	}

    @Override
    public boolean hasChildNode(String name) {
		return false;
	}

    /**
     * Returns a mutable child node with the given name. If the named
     * child node has already been modified, i.e. there's an entry for
     * it in the {@link #nodes} map, then that child instance is returned
     * directly. Otherwise a new mutable child instance is created based
     * on the (possibly non-existent) respective child node of the base
     * state, added to the {@link #nodes} map and returned.
     */
    MutableNodeState getMutableChildNode(String name) {
		return null;
	}

    @Override
    public NodeState getChildNode(String name) {
		return null;
	}

    @Override @Nonnull
    public Iterable<String> getChildNodeNames() {
		return null;
	}

    @Override @Nonnull
    public Iterable<? extends ChildNodeEntry> getChildNodeEntries() {
		return null;
	}

    @Override
    public boolean compareAgainstBaseState(NodeState base, NodeStateDiff diff) {
		return false;
	}

    @Override @Nonnull
    public NodeBuilder builder() {
		return null;
	}

}
